# Generated by Django 5.2.3 on 2025-06-21 14:42

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('blog', '0026_remove_blogpost_blog_post_status_ix_and_more'),
    ]

    operations = [
        migrations.AlterModelManagers(
            name='blogtag',
            managers=[
            ],
        ),
    ]



from django.conf import settings
from django.test import TestCase

from blog.factories.post import BlogPostFactory
from blog.factories.tag import BlogTagFactory
from blog.models.tag import BlogTag

languages = [
    lang["code"] for lang in settings.PARLER_LANGUAGES[settings.SITE_ID]
]
default_language = settings.PARLER_DEFAULT_LANGUAGE_CODE


class BlogTagModelTestCase(TestCase):
    def setUp(self):
        self.tag = BlogTagFactory(active=True)
        self.tag.sort_order = 0

    def test_fields(self):
        self.assertTrue(self.tag.active)

    def test_str_representation(self):
        tag_name = (
            self.tag.safe_translation_getter("name", any_language=True)
            or "Unnamed Tag"
        )
        self.assertEqual(
            str(self.tag),
            f"{tag_name} ({'Active' if self.tag.active else 'Inactive'})",
        )

    def test_get_ordering_queryset(self):
        BlogTag.objects.all().delete()

        tags = [
            BlogTagFactory(sort_order=1, active=True),
            BlogTagFactory(sort_order=2, active=True),
            BlogTagFactory(sort_order=3, active=True),
        ]

        for tag in tags:
            tag.save()

        ordered_tags = BlogTag.objects.all().order_by("sort_order")

        ordered_sort_orders = [tag.sort_order for tag in ordered_tags]

        self.assertEqual(ordered_sort_orders, [1, 2, 3])

    def test_get_tag_posts_count(self):
        post = BlogPostFactory(
            num_tags=0,
            num_comments=0,
        )
        post.tags.set([self.tag])
        self.assertEqual(self.tag.get_posts_count, 1)



from extra_settings.models import Setting
from measurement.measures import Weight

from core.units import WeightUnits


def zero_weight():
    return Weight(kg=0)


def convert_weight(weight: Weight, unit: str):
    converted_weight = getattr(weight, unit)
    weight = Weight(**{unit: converted_weight})
    weight.value = round(weight.value, 3)
    return weight


def get_default_weight_unit():
    return Setting.get("DEFAULT_WEIGHT_UNIT", default=WeightUnits.KG)


def convert_weight_to_default_weight_unit(weight: Weight):
    default_unit = get_default_weight_unit()
    if weight is not None:
        if weight.unit != default_unit:
            weight = convert_weight(weight, default_unit)
        else:
            weight.value = round(weight.value, 3)
    return weight



# Generated by Django 5.2.1 on 2025-05-14 20:33

import django.contrib.postgres.indexes
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('country', '0005_alter_country_alpha_2_alter_country_alpha_3_and_more'),
    ]

    operations = [
        migrations.RemoveIndex(
            model_name='country',
            name='country_cou_alpha_2_398383_idx',
        ),
        migrations.RemoveIndex(
            model_name='country',
            name='country_cou_alpha_3_c813cd_idx',
        ),
        migrations.RenameIndex(
            model_name='country',
            new_name='country_created_at_ix',
            old_name='country_created_at_idx',
        ),
        migrations.RenameIndex(
            model_name='country',
            new_name='country_updated_at_ix',
            old_name='country_updated_at_idx',
        ),
        migrations.RenameIndex(
            model_name='country',
            new_name='country_sort_order_ix',
            old_name='country_sort_order_idx',
        ),
        migrations.AddIndex(
            model_name='country',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['alpha_2'], name='country_alpha_2_ix'),
        ),
        migrations.AddIndex(
            model_name='country',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['alpha_3'], name='country_alpha_3_ix'),
        ),
        migrations.AddIndex(
            model_name='country',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['iso_cc'], name='country_iso_cc_ix'),
        ),
        migrations.AddIndex(
            model_name='country',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['phone_code'], name='country_phone_code_ix'),
        ),
    ]



import pytest
from datetime import timedelta
from unittest.mock import Mock, patch

from django.contrib.admin.sites import AdminSite
from django.contrib.auth import get_user_model
from django.contrib.auth.models import Group
from django.test import RequestFactory
from django.utils import timezone

from user.admin import (
    GroupAdmin,
    UserAdmin,
    UserAddressAdmin,
    SubscriptionTopicAdmin,
    UserSubscriptionAdmin,
    SubscriptionCountFilter,
    AddressCountFilter,
    UserStatusFilter,
    SocialMediaFilter,
    TopicCategoryFilter,
    UserAddressInline,
    UserSubscriptionInline,
)
from user.factories import (
    UserAccountFactory,
    UserAddressFactory,
    SubscriptionTopicFactory,
    UserSubscriptionFactory,
)
from user.models import (
    UserAccount,
    UserAddress,
    SubscriptionTopic,
    UserSubscription,
)
from country.factories import CountryFactory
from region.factories import RegionFactory

User = get_user_model()


@pytest.fixture
def admin_request():
    factory = RequestFactory()
    request = factory.get("/admin/user/useraccount/")
    request.user = Mock()
    request.user.is_authenticated = True
    request.user.is_staff = True
    request.user.is_superuser = True
    return request


@pytest.mark.django_db
class TestCustomFilters:
    def test_subscription_count_filter_queryset(self, admin_request):
        admin = UserAdmin(UserAccount, AdminSite())
        admin_filter = SubscriptionCountFilter(
            admin_request,
            {"subscription_count_from": "1", "subscription_count_to": "5"},
            UserAccount,
            UserAdmin,
        )

        user1 = UserAccountFactory()
        user2 = UserAccountFactory()

        topic = SubscriptionTopicFactory()
        UserSubscriptionFactory(user=user1, topic=topic)
        UserSubscriptionFactory(user=user2, topic=topic)
        UserSubscriptionFactory(user=user2, topic=SubscriptionTopicFactory())

        queryset = admin.get_queryset(admin_request)
        filtered_queryset = admin_filter.queryset(admin_request, queryset)

        assert filtered_queryset is not None

    def test_subscription_count_filter_expected_parameters(self):
        admin_filter = SubscriptionCountFilter(None, {}, UserAccount, UserAdmin)
        expected = ["subscription_count_from", "subscription_count_to"]
        assert admin_filter.expected_parameters() == expected

    def test_address_count_filter_queryset(self, admin_request):
        admin = UserAdmin(UserAccount, AdminSite())
        admin_filter = AddressCountFilter(
            admin_request,
            {"address_count_from": "1", "address_count_to": "3"},
            UserAccount,
            UserAdmin,
        )

        user1 = UserAccountFactory()
        user2 = UserAccountFactory()
        country = CountryFactory()

        UserAddressFactory(user=user1, country=country)
        UserAddressFactory(user=user2, country=country)
        UserAddressFactory(user=user2, country=country)

        queryset = admin.get_queryset(admin_request)
        filtered_queryset = admin_filter.queryset(admin_request, queryset)

        assert filtered_queryset is not None

    def test_address_count_filter_expected_parameters(self):
        admin_filter = AddressCountFilter(None, {}, UserAccount, UserAdmin)
        expected = ["address_count_from", "address_count_to"]
        assert admin_filter.expected_parameters() == expected

    def test_user_status_filter_lookups(self, admin_request):
        admin_filter = UserStatusFilter(
            admin_request, {}, UserAccount, UserAdmin
        )
        lookups = admin_filter.lookups(admin_request, UserAdmin)

        expected_keys = [
            "active_staff",
            "active_regular",
            "inactive",
            "superuser",
            "with_subscriptions",
            "no_subscriptions",
        ]
        lookup_keys = [lookup[0] for lookup in lookups]

        for key in expected_keys:
            assert key in lookup_keys

    def test_user_status_filter_active_staff(self, admin_request):
        staff_user = UserAccountFactory(is_active=True, is_staff=True)
        regular_user = UserAccountFactory(is_active=True, is_staff=False)
        inactive_user = UserAccountFactory(is_active=False, is_staff=True)

        admin_filter = UserStatusFilter(
            admin_request, {}, UserAccount, UserAdmin
        )

        admin_filter.value = lambda: "active_staff"

        queryset = UserAccount.objects.all()
        filtered_queryset = admin_filter.queryset(admin_request, queryset)

        result_users = list(filtered_queryset)

        assert staff_user in result_users
        assert regular_user not in result_users
        assert inactive_user not in result_users

    def test_user_status_filter_superuser(self, admin_request):
        superuser = UserAccountFactory(is_superuser=True)
        regular_user = UserAccountFactory(is_superuser=False)

        admin_filter = UserStatusFilter(
            admin_request, {}, UserAccount, UserAdmin
        )
        admin_filter.value = lambda: "superuser"

        queryset = UserAccount.objects.all()
        filtered_queryset = admin_filter.queryset(admin_request, queryset)

        result_users = list(filtered_queryset)
        assert superuser in result_users
        assert regular_user not in result_users

    def test_social_media_filter_lookups(self, admin_request):
        admin_filter = SocialMediaFilter(
            admin_request, {}, UserAccount, UserAdmin
        )
        lookups = admin_filter.lookups(admin_request, UserAdmin)

        expected_keys = ["has_website", "has_social", "no_social"]
        lookup_keys = [lookup[0] for lookup in lookups]

        for key in expected_keys:
            assert key in lookup_keys

    def test_social_media_filter_has_website(self, admin_request):
        user_with_website = UserAccountFactory(website="https://example.com")
        user_without_website = UserAccountFactory(website="")

        admin_filter = SocialMediaFilter(
            admin_request, {}, UserAccount, UserAdmin
        )
        admin_filter.value = lambda: "has_website"

        queryset = UserAccount.objects.all()
        filtered_queryset = admin_filter.queryset(admin_request, queryset)

        result_users = list(filtered_queryset)
        assert user_with_website in result_users
        assert user_without_website not in result_users

    def test_social_media_filter_has_social(self, admin_request):
        user_with_twitter = UserAccountFactory(twitter="@user")
        user_with_linkedin = UserAccountFactory(linkedin="linkedin.com/in/user")
        user_without_social = UserAccountFactory(
            website="",
            twitter="",
            linkedin="",
            facebook="",
            instagram="",
            youtube="",
            github="",
        )

        admin_filter = SocialMediaFilter(
            admin_request, {}, UserAccount, UserAdmin
        )
        admin_filter.value = lambda: "has_social"

        queryset = UserAccount.objects.all()
        filtered_queryset = admin_filter.queryset(admin_request, queryset)

        result_users = list(filtered_queryset)
        assert user_with_twitter in result_users
        assert user_with_linkedin in result_users
        assert user_without_social not in result_users

    def test_topic_category_filter_lookups(self, admin_request):
        admin_filter = TopicCategoryFilter(
            admin_request, {}, UserSubscription, UserSubscriptionAdmin
        )
        lookups = admin_filter.lookups(admin_request, UserSubscriptionAdmin)

        assert len(lookups) > 0
        assert all(len(choice) == 2 for choice in lookups)

    def test_topic_category_filter_queryset(self, admin_request):
        topic1 = SubscriptionTopicFactory(
            category=SubscriptionTopic.TopicCategory.NEWSLETTER
        )
        topic2 = SubscriptionTopicFactory(
            category=SubscriptionTopic.TopicCategory.MARKETING
        )

        user = UserAccountFactory()
        subscription1 = UserSubscriptionFactory(user=user, topic=topic1)
        subscription2 = UserSubscriptionFactory(user=user, topic=topic2)

        admin_filter = TopicCategoryFilter(
            admin_request, {}, UserSubscription, UserSubscriptionAdmin
        )
        admin_filter.value = lambda: SubscriptionTopic.TopicCategory.NEWSLETTER

        queryset = UserSubscription.objects.all()
        filtered_queryset = admin_filter.queryset(admin_request, queryset)

        result_subscriptions = list(filtered_queryset)
        assert subscription1 in result_subscriptions
        assert subscription2 not in result_subscriptions


@pytest.mark.django_db
class TestInlineClasses:
    def test_user_address_inline_configuration(self):
        inline = UserAddressInline(UserAccount, AdminSite())

        assert inline.model == UserAddress
        assert inline.extra == 0
        assert "title" in inline.fields
        assert "first_name" in inline.fields
        assert "is_main" in inline.fields
        assert "created_at" in inline.readonly_fields
        assert inline.show_change_link is True
        assert inline.tab is True

    def test_user_subscription_inline_configuration(self):
        inline = UserSubscriptionInline(UserAccount, AdminSite())

        assert inline.model == UserSubscription
        assert inline.extra == 0
        assert "topic" in inline.fields
        assert "status" in inline.fields
        assert "subscribed_at" in inline.readonly_fields
        assert "unsubscribed_at" in inline.readonly_fields
        assert inline.show_change_link is True


@pytest.mark.django_db
class TestGroupAdmin:
    def test_group_admin_inheritance(self):
        site = AdminSite()
        admin = GroupAdmin(Group, site)

        assert hasattr(admin, "list_display")
        assert hasattr(admin, "search_fields")


@pytest.mark.django_db
class TestUserAdmin:
    def setUp(self):
        self.site = AdminSite()
        self.admin = UserAdmin(UserAccount, self.site)
        self.country = CountryFactory()
        self.region = RegionFactory(country=self.country)

    def test_user_admin_configuration(self, admin_request):
        admin = UserAdmin(UserAccount, AdminSite())

        assert admin.compressed_fields is True
        assert admin.warn_unsaved_form is True
        assert admin.list_fullwidth is True
        assert admin.list_filter_submit is True
        assert admin.list_filter_sheet is True
        assert admin.save_on_top is True
        assert admin.list_per_page == 25

    def test_list_display(self, admin_request):
        admin = UserAdmin(UserAccount, AdminSite())
        expected_fields = [
            "user_profile_display",
            "contact_info_display",
            "location_display",
            "user_status_badges",
            "social_links_display",
            "engagement_metrics",
            "last_activity",
            "created_at",
        ]
        assert admin.list_display == expected_fields

    def test_user_profile_display(self, admin_request):
        admin = UserAdmin(UserAccount, AdminSite())
        user = UserAccountFactory(
            first_name="John",
            last_name="Doe",
            email="john@example.com",
            username="johndoe",
        )

        result = admin.user_profile_display(user)

        assert "John Doe" in result
        assert "john@example.com" in result

    def test_contact_info_display(self, admin_request):
        admin = UserAdmin(UserAccount, AdminSite())
        user = UserAccountFactory(phone="+1234567890", email="test@example.com")

        result = admin.contact_info_display(user)

        assert "+1234567890" in result
        assert "📞" in result

    def test_location_display(self, admin_request):
        admin = UserAdmin(UserAccount, AdminSite())
        country = CountryFactory()
        region = RegionFactory(country=country)
        user = UserAccountFactory(
            city="New York", country=country, region=region
        )

        result = admin.location_display(user)

        assert "New York" in result
        assert "📍" in result or "map-pin" in result

    def test_user_status_badges(self, admin_request):
        admin = UserAdmin(UserAccount, AdminSite())

        staff_user = UserAccountFactory(
            is_active=True, is_staff=True, is_superuser=False
        )
        result = admin.user_status_badges(staff_user)
        assert "Active" in result
        assert "Staff" in result

        super_user = UserAccountFactory(is_active=True, is_superuser=True)
        result = admin.user_status_badges(super_user)
        assert "Super" in result

    def test_social_links_display(self, admin_request):
        admin = UserAdmin(UserAccount, AdminSite())
        user = UserAccountFactory(
            website="https://example.com",
            twitter="@user",
            github="github.com/user",
        )

        result = admin.social_links_display(user)

        assert "🌐" in result
        assert "🐦" in result
        assert "💻" in result

    def test_engagement_metrics(self, admin_request):
        admin = UserAdmin(UserAccount, AdminSite())
        user = UserAccountFactory()

        result = admin.engagement_metrics(user)

        assert (
            "Subscriptions" in result or "Addresses" in result or "0" in result
        )

    def test_last_activity(self, admin_request):
        admin = UserAdmin(UserAccount, AdminSite())
        user = UserAccountFactory()

        result = admin.last_activity(user)

        assert "Updated:" in result

    def test_social_links_summary(self, admin_request):
        admin = UserAdmin(UserAccount, AdminSite())
        user = UserAccountFactory(
            website="https://example.com",
            linkedin="linkedin.com/in/user",
            twitter="@user",
        )

        result = admin.social_links_summary(user)

        assert "Website" in result
        assert "LinkedIn" in result
        assert "Twitter" in result

    def test_subscription_summary(self, admin_request):
        admin = UserAdmin(UserAccount, AdminSite())
        user = UserAccountFactory()

        topic = SubscriptionTopicFactory()
        UserSubscriptionFactory(user=user, topic=topic)

        result = admin.subscription_summary(user)

        assert "Active:" in result

    def test_address_summary(self, admin_request):
        admin = UserAdmin(UserAccount, AdminSite())
        user = UserAccountFactory()
        country = CountryFactory()

        UserAddressFactory(user=user, country=country)

        result = admin.address_summary(user)

        assert "total" in result.lower()


@pytest.mark.django_db
class TestUserAddressAdmin:
    def test_user_address_admin_configuration(self):
        admin = UserAddressAdmin(UserAddress, AdminSite())

        assert admin.compressed_fields is True
        assert admin.warn_unsaved_form is True
        assert admin.list_fullwidth is True

        expected_fields = [
            "address_display",
            "contact_person",
            "location_info",
            "main_address_badge",
            "contact_numbers",
            "created_at",
        ]
        assert admin.list_display == expected_fields

    def test_address_display(self):
        admin = UserAddressAdmin(UserAddress, AdminSite())
        country = CountryFactory()
        address = UserAddressFactory(
            title="Home", street="123 Main St", city="New York", country=country
        )

        result = admin.address_display(address)

        assert "Home" in result or "123 Main St" in result
        assert "New York" in result

    def test_contact_person(self):
        admin = UserAddressAdmin(UserAddress, AdminSite())
        address = UserAddressFactory(first_name="John", last_name="Doe")

        result = admin.contact_person(address)

        assert "John" in result
        assert "Doe" in result

    def test_location_info(self):
        admin = UserAddressAdmin(UserAddress, AdminSite())
        country = CountryFactory()
        region = RegionFactory(country=country)
        address = UserAddressFactory(
            city="New York", country=country, region=region
        )

        result = admin.location_info(address)

        assert "New York" in result

    def test_main_address_badge(self):
        admin = UserAddressAdmin(UserAddress, AdminSite())

        main_address = UserAddressFactory(is_main=True)
        result = admin.main_address_badge(main_address)
        assert "Main" in result or "Primary" in result

        regular_address = UserAddressFactory(is_main=False)
        result = admin.main_address_badge(regular_address)
        assert "Main" not in result or "Secondary" in result

    def test_contact_numbers(self):
        admin = UserAddressAdmin(UserAddress, AdminSite())
        address = UserAddressFactory(
            phone="+1234567890", mobile_phone="+9876543210"
        )

        result = admin.contact_numbers(address)

        assert "+1234567890" in result or "+9876543210" in result


@pytest.mark.django_db
class TestSubscriptionTopicAdmin:
    def test_subscription_topic_admin_configuration(self):
        admin = SubscriptionTopicAdmin(SubscriptionTopic, AdminSite())

        assert admin.compressed_fields is True
        assert admin.warn_unsaved_form is True
        assert admin.list_filter_submit is True

        expected_fields = [
            "name_display",
            "category_badge",
            "active_status",
            "is_active",
            "settings_badges",
            "subscriber_metrics",
            "created_at",
        ]
        assert admin.list_display == expected_fields

    def test_name_display(self):
        admin = SubscriptionTopicAdmin(SubscriptionTopic, AdminSite())
        topic = SubscriptionTopicFactory()
        topic.set_current_language("en")
        topic.name = "Newsletter"
        topic.save()

        result = admin.name_display(topic)

        assert "Newsletter" in result

    def test_category_badge(self):
        admin = SubscriptionTopicAdmin(SubscriptionTopic, AdminSite())
        topic = SubscriptionTopicFactory(
            category=SubscriptionTopic.TopicCategory.NEWSLETTER
        )

        result = admin.category_badge(topic)

        assert "Newsletter" in result or "NEWSLETTER" in result

    def test_active_status(self):
        admin = SubscriptionTopicAdmin(SubscriptionTopic, AdminSite())

        active_topic = SubscriptionTopicFactory(is_active=True)
        result = admin.active_status(active_topic)
        assert "Active" in result

        inactive_topic = SubscriptionTopicFactory(is_active=False)
        result = admin.active_status(inactive_topic)
        assert "Inactive" in result

    def test_settings_badges(self):
        admin = SubscriptionTopicAdmin(SubscriptionTopic, AdminSite())
        topic = SubscriptionTopicFactory(
            is_default=True, requires_confirmation=True
        )

        result = admin.settings_badges(topic)

        assert "Default" in result
        assert "Confirm" in result

    def test_subscriber_metrics(self):
        admin = SubscriptionTopicAdmin(SubscriptionTopic, AdminSite())
        topic = SubscriptionTopicFactory()

        user1 = UserAccountFactory()
        user2 = UserAccountFactory()
        UserSubscriptionFactory(topic=topic, user=user1)
        UserSubscriptionFactory(topic=topic, user=user2)

        result = admin.subscriber_metrics(topic)

        assert "\u2713" in result or "\U0001f465" in result


@pytest.mark.django_db
class TestUserSubscriptionAdmin:
    def test_user_subscription_admin_configuration(self):
        admin = UserSubscriptionAdmin(UserSubscription, AdminSite())

        assert admin.compressed_fields is True
        assert admin.warn_unsaved_form is True
        assert admin.list_fullwidth is True

        expected_fields = [
            "subscription_info",
            "user_info",
            "topic_info",
            "status_display",
            "status",
            "subscription_dates",
            "created_at",
        ]
        assert admin.list_display == expected_fields

    def test_subscription_info(self):
        admin = UserSubscriptionAdmin(UserSubscription, AdminSite())
        user = UserAccountFactory()
        topic = SubscriptionTopicFactory()
        subscription = UserSubscriptionFactory(user=user, topic=topic)

        result = admin.subscription_info(subscription)

        assert (
            "subscription" in result.lower() or str(subscription.id) in result
        )

    def test_user_info(self):
        admin = UserSubscriptionAdmin(UserSubscription, AdminSite())
        user = UserAccountFactory(email="test@example.com", username="testuser")
        topic = SubscriptionTopicFactory()
        subscription = UserSubscriptionFactory(user=user, topic=topic)

        result = admin.user_info(subscription)

        assert "test@example.com" in result or "testuser" in result

    def test_topic_info(self):
        admin = UserSubscriptionAdmin(UserSubscription, AdminSite())
        user = UserAccountFactory()
        topic = SubscriptionTopicFactory()
        topic.set_current_language("en")
        topic.name = "Weekly Newsletter"
        topic.save()
        subscription = UserSubscriptionFactory(user=user, topic=topic)

        result = admin.topic_info(subscription)

        assert "Weekly Newsletter" in result or "Newsletter" in result

    def test_status_display(self):
        admin = UserSubscriptionAdmin(UserSubscription, AdminSite())
        user = UserAccountFactory()
        topic1 = SubscriptionTopicFactory()
        topic2 = SubscriptionTopicFactory()

        active_sub = UserSubscriptionFactory(
            user=user,
            topic=topic1,
            status=UserSubscription.SubscriptionStatus.ACTIVE,
        )
        result = admin.status_display(active_sub)
        assert "Active" in result

        unsubscribed_sub = UserSubscriptionFactory(
            user=user,
            topic=topic2,
            status=UserSubscription.SubscriptionStatus.UNSUBSCRIBED,
        )
        result = admin.status_display(unsubscribed_sub)
        assert "Unsubscribed" in result

    def test_subscription_dates(self):
        admin = UserSubscriptionAdmin(UserSubscription, AdminSite())
        user = UserAccountFactory()
        topic = SubscriptionTopicFactory()
        subscription = UserSubscriptionFactory(
            user=user,
            topic=topic,
            subscribed_at=timezone.now() - timedelta(days=30),
        )

        result = admin.subscription_dates(subscription)

        assert "Subscribed" in result or "ago" in result

    @patch.object(UserSubscriptionAdmin, "message_user")
    def test_activate_subscriptions_action(
        self, mock_message_user, admin_request
    ):
        admin = UserSubscriptionAdmin(UserSubscription, AdminSite())

        user = UserAccountFactory()
        topic = SubscriptionTopicFactory()
        subscription1 = UserSubscriptionFactory(
            user=user,
            topic=topic,
            status=UserSubscription.SubscriptionStatus.UNSUBSCRIBED,
        )
        subscription2 = UserSubscriptionFactory(
            user=UserAccountFactory(),
            topic=topic,
            status=UserSubscription.SubscriptionStatus.UNSUBSCRIBED,
        )

        queryset = UserSubscription.objects.filter(
            id__in=[subscription1.id, subscription2.id]
        )

        admin.activate_subscriptions(admin_request, queryset)

        subscription1.refresh_from_db()
        subscription2.refresh_from_db()
        assert (
            subscription1.status == UserSubscription.SubscriptionStatus.ACTIVE
        )
        assert (
            subscription2.status == UserSubscription.SubscriptionStatus.ACTIVE
        )

        mock_message_user.assert_called_once()

    @patch.object(UserSubscriptionAdmin, "message_user")
    def test_deactivate_subscriptions_action(
        self, mock_message_user, admin_request
    ):
        admin = UserSubscriptionAdmin(UserSubscription, AdminSite())

        user = UserAccountFactory()
        topic = SubscriptionTopicFactory()
        subscription1 = UserSubscriptionFactory(
            user=user,
            topic=topic,
            status=UserSubscription.SubscriptionStatus.ACTIVE,
        )
        subscription2 = UserSubscriptionFactory(
            user=UserAccountFactory(),
            topic=topic,
            status=UserSubscription.SubscriptionStatus.ACTIVE,
        )

        queryset = UserSubscription.objects.filter(
            id__in=[subscription1.id, subscription2.id]
        )

        admin.deactivate_subscriptions(admin_request, queryset)

        subscription1.refresh_from_db()
        subscription2.refresh_from_db()
        assert (
            subscription1.status
            == UserSubscription.SubscriptionStatus.UNSUBSCRIBED
        )
        assert (
            subscription2.status
            == UserSubscription.SubscriptionStatus.UNSUBSCRIBED
        )

        mock_message_user.assert_called_once()


@pytest.mark.django_db
class TestIntegration:
    def test_admin_classes_registered(self):
        from django.contrib import admin

        assert UserAccount in admin.site._registry
        assert UserAddress in admin.site._registry
        assert SubscriptionTopic in admin.site._registry
        assert UserSubscription in admin.site._registry
        assert Group in admin.site._registry

    def test_queryset_optimization(self):
        admin = UserAdmin(UserAccount, AdminSite())

        assert "country" in admin.list_select_related
        assert "region" in admin.list_select_related

    def test_search_functionality(self):
        admin = UserAdmin(UserAccount, AdminSite())

        expected_search_fields = [
            "email",
            "username",
            "first_name",
            "last_name",
            "phone",
            "city",
            "address",
            "bio",
        ]

        for field in expected_search_fields:
            assert field in admin.search_fields



from django.db import models
from django.utils.translation import gettext_lazy as _


class OrderDocumentTypeEnum(models.TextChoices):
    RECEIPT = "RECEIPT", _("Receipt")
    INVOICE = "INVOICE", _("Invoice")

    PROFORMA = "PROFORMA", _("Proforma Invoice")
    SHIPPING_LABEL = "SHIPPING_LABEL", _("Shipping Label")
    RETURN_LABEL = "RETURN_LABEL", _("Return Label")
    CREDIT_NOTE = "CREDIT_NOTE", _("Credit Note")

    @classmethod
    def get_document_types_for_status(cls, status: str) -> list[str]:
        from order.enum.status import OrderStatus  # noqa: PLC0415

        if status in [OrderStatus.PENDING, OrderStatus.PROCESSING]:
            return [cls.RECEIPT, cls.INVOICE, cls.PROFORMA]
        elif status in [OrderStatus.SHIPPED, OrderStatus.DELIVERED]:
            return [cls.RECEIPT, cls.INVOICE, cls.SHIPPING_LABEL]
        elif status == OrderStatus.RETURNED:
            return [cls.RECEIPT, cls.INVOICE, cls.RETURN_LABEL]
        elif status == OrderStatus.REFUNDED:
            return [cls.RECEIPT, cls.INVOICE, cls.CREDIT_NOTE]
        else:
            return [cls.RECEIPT, cls.INVOICE]



import importlib
import random

import factory
from django.apps import apps
from django.contrib.auth import get_user_model

from core.enum import FloorChoicesEnum, LocationChoicesEnum
from user.models.address import UserAddress

User = get_user_model()


def get_or_create_user():
    if User.objects.exists():
        user = User.objects.order_by("?").first()
    else:
        user_factory_module = importlib.import_module("user.factories.account")
        user_factory_class = user_factory_module.UserAccountFactory
        user = user_factory_class.create()
    return user


def get_or_create_country():
    if apps.get_model("country", "Country").objects.exists():
        return (
            apps.get_model("country", "Country").objects.order_by("?").first()
        )
    else:
        country_factory_module = importlib.import_module("country.factories")
        country_factory_class = country_factory_module.CountryFactory
        return country_factory_class.create()


def get_or_create_region():
    if apps.get_model("region", "Region").objects.exists():
        return apps.get_model("region", "Region").objects.order_by("?").first()
    else:
        region_factory_module = importlib.import_module("region.factories")
        region_factory_class = region_factory_module.RegionFactory
        return region_factory_class.create()


class UserAddressFactory(factory.django.DjangoModelFactory):
    user = factory.LazyFunction(get_or_create_user)
    title = factory.Faker("word")
    first_name = factory.Faker("first_name")
    last_name = factory.Faker("last_name")
    street = factory.Faker("street_name")
    street_number = factory.Faker("building_number")
    city = factory.Faker("city")
    zipcode = factory.Faker("postcode")
    country = factory.LazyFunction(get_or_create_country)
    region = factory.LazyFunction(get_or_create_region)
    floor = factory.LazyFunction(
        lambda: random.choice([s[0] for s in FloorChoicesEnum.choices])
    )
    location_type = factory.LazyFunction(
        lambda: random.choice([s[0] for s in LocationChoicesEnum.choices])
    )
    phone = factory.Faker("phone_number")
    mobile_phone = factory.Faker("phone_number")
    notes = factory.Faker("sentence")
    is_main = factory.Faker("boolean")

    class Meta:
        model = UserAddress
        django_get_or_create = ("user", "title")



import os
import logging


class HostnameFilter(logging.Filter):
    def filter(self, record):
        record.hostname = os.getenv("HOSTNAME", "unknown")
        return True


def get_logger(name: str) -> logging.Logger:
    return logging.getLogger(name)



from django.urls import path
from rest_framework.urlpatterns import format_suffix_patterns

from country.views import CountryViewSet

urlpatterns = [
    path(
        "country",
        CountryViewSet.as_view({"get": "list", "post": "create"}),
        name="country-list",
    ),
    path(
        "country/<str:pk>",
        CountryViewSet.as_view(
            {
                "get": "retrieve",
                "put": "update",
                "patch": "partial_update",
                "delete": "destroy",
            }
        ),
        name="country-detail",
    ),
]

urlpatterns = format_suffix_patterns(urlpatterns)



from django.utils.translation import gettext_lazy as _
from djmoney.contrib.django_rest_framework import MoneyField
from djmoney.money import Money
from drf_spectacular.utils import extend_schema_field
from measurement.measures import Weight
from parler_rest.serializers import TranslatableModelSerializer
from rest_framework import serializers
from rest_framework.relations import PrimaryKeyRelatedField

from core.api.schema import generate_schema_multi_lang
from core.api.serializers import MeasurementSerializerField
from core.utils.serializers import TranslatedFieldExtended
from product.models.category import ProductCategory
from product.models.product import Product
from vat.models import Vat


@extend_schema_field(generate_schema_multi_lang(Product))
class TranslatedFieldsFieldExtend(TranslatedFieldExtended):
    pass


class ProductSerializer(
    TranslatableModelSerializer, serializers.ModelSerializer[Product]
):
    translations = TranslatedFieldsFieldExtend(shared_model=Product)
    category = PrimaryKeyRelatedField(queryset=ProductCategory.objects.all())
    vat = PrimaryKeyRelatedField(queryset=Vat.objects.all())
    price = MoneyField(max_digits=11, decimal_places=2)
    final_price = MoneyField(max_digits=11, decimal_places=2, read_only=True)
    discount_value = MoneyField(max_digits=11, decimal_places=2, read_only=True)
    vat_value = MoneyField(max_digits=11, decimal_places=2, read_only=True)
    weight = MeasurementSerializerField(
        measurement=Weight, required=False, allow_null=True
    )

    class Meta:
        model = Product
        fields = (
            "id",
            "translations",
            "slug",
            "category",
            "price",
            "vat",
            "view_count",
            "stock",
            "active",
            "weight",
            "seo_title",
            "seo_description",
            "seo_keywords",
            "discount_percent",
            "created_at",
            "updated_at",
            "uuid",
            "discount_value",
            "price_save_percent",
            "vat_percent",
            "vat_value",
            "final_price",
            "main_image_path",
            "review_average",
            "review_count",
            "likes_count",
            "created_at",
            "updated_at",
            "uuid",
        )

        read_only_fields = (
            "id",
            "final_price",
            "discount_value",
            "price_save_percent",
            "vat_percent",
            "vat_value",
            "final_price",
            "main_image_path",
            "review_average",
            "review_count",
            "likes_count",
            "view_count",
            "created_at",
            "updated_at",
            "uuid",
        )


class ProductDetailSerializer(ProductSerializer):
    class Meta(ProductSerializer.Meta):
        fields = (*ProductSerializer.Meta.fields,)


class ProductWriteSerializer(
    TranslatableModelSerializer, serializers.ModelSerializer[Product]
):
    category = PrimaryKeyRelatedField(queryset=ProductCategory.objects.all())
    vat = PrimaryKeyRelatedField(queryset=Vat.objects.all())
    price = MoneyField(max_digits=11, decimal_places=2)
    weight = MeasurementSerializerField(
        measurement=Weight, required=False, allow_null=True
    )
    translations = TranslatedFieldsFieldExtend(shared_model=Product)

    def validate_price(self, value: Money) -> Money:
        if value.amount <= 0:
            raise serializers.ValidationError(
                _("Price must be greater than zero.")
            )
        if value.amount > 99999:
            raise serializers.ValidationError(_("Price cannot exceed 99,999."))
        return value

    def validate_stock(self, value: int) -> int:
        if value < 0:
            raise serializers.ValidationError(_("Stock cannot be negative."))
        return value

    def validate_discount_percent(self, value: int) -> int:
        if value < 0 or value > 100:
            raise serializers.ValidationError(
                _("Discount percent must be between 0 and 100.")
            )
        return value

    def validate_slug(self, value: str) -> str:
        if not value:
            raise serializers.ValidationError(_("Slug is required."))

        queryset = Product.objects.filter(slug=value)
        if self.instance:
            queryset = queryset.exclude(pk=self.instance.pk)

        if queryset.exists():
            raise serializers.ValidationError(
                _("A post with this slug already exists.")
            )

        return value

    class Meta:
        model = Product
        fields = (
            "translations",
            "slug",
            "category",
            "price",
            "vat",
            "stock",
            "weight",
            "discount_percent",
            "seo_title",
            "seo_description",
            "seo_keywords",
            "active",
        )



from __future__ import annotations

from datetime import timedelta

from django.db.models import Count, Q
from django.utils import timezone
from parler.managers import TranslatableManager, TranslatableQuerySet

from product.enum.review import ReviewStatus


class EnhancedReviewQuerySet(TranslatableQuerySet):
    def approved(self):
        return self.filter(status=ReviewStatus.TRUE)

    def pending(self):
        return self.filter(status=ReviewStatus.NEW)

    def rejected(self):
        return self.filter(status=ReviewStatus.FALSE)

    def published(self):
        return self.filter(is_published=True)

    def visible(self):
        return self.filter(is_published=True)

    def for_product(self, product):
        if hasattr(product, "pk"):
            return self.filter(product=product.pk)
        return self.filter(product=product)

    def for_user(self, user):
        if hasattr(user, "pk"):
            return self.filter(user=user.pk)
        return self.filter(user=user)

    def by_rate(self, rate):
        return self.filter(rate=rate)

    def high_rated(self, min_rate=4):
        return self.filter(rate__gte=min_rate)

    def low_rated(self, max_rate=2):
        return self.filter(rate__lte=max_rate)

    def recent(self, days=30):
        cutoff_date = timezone.now() - timedelta(days=days)
        return self.filter(created_at__gte=cutoff_date)

    def with_comments(self):
        return self.filter(translations__comment__isnull=False).distinct()

    def without_comments(self):
        return self.filter(
            Q(translations__comment__isnull=True)
            | Q(translations__comment__exact="")
        ).distinct()

    def with_product_details(self):
        return self.select_related("product", "user").prefetch_related(
            "product__translations", "product__images", "translations"
        )

    def annotate_user_review_count(self):
        return self.annotate(user_review_count=Count("user__product_reviews"))


class ProductReviewManager(TranslatableManager):
    def get_queryset(self) -> EnhancedReviewQuerySet:
        return EnhancedReviewQuerySet(self.model, using=self._db)

    def approved(self):
        return self.get_queryset().approved()

    def pending(self):
        return self.get_queryset().pending()

    def rejected(self):
        return self.get_queryset().rejected()

    def published(self):
        return self.get_queryset().published()

    def visible(self):
        return self.get_queryset().visible()

    def for_product(self, product):
        return self.get_queryset().for_product(product)

    def for_user(self, user):
        return self.get_queryset().for_user(user)

    def by_rate(self, rate):
        return self.get_queryset().by_rate(rate)

    def high_rated(self, min_rate=4):
        return self.get_queryset().high_rated(min_rate)

    def low_rated(self, max_rate=2):
        return self.get_queryset().low_rated(max_rate)

    def recent(self, days=30):
        return self.get_queryset().recent(days)

    def with_comments(self):
        return self.get_queryset().with_comments()

    def without_comments(self):
        return self.get_queryset().without_comments()

    def with_product_details(self):
        return self.get_queryset().with_product_details()

    def annotate_user_review_count(self):
        return self.get_queryset().annotate_user_review_count()



import logging
from collections.abc import Iterable
from json import loads
from typing import Any, TypedDict

from django.conf import settings
from django.core.serializers import serialize
from django.db import models
from django.db.models.fields import Field
from meilisearch.models.task import Task, TaskInfo

from meili._client import client as _client
from meili.dataclasses import MeiliIndexSettings
from meili.querysets import IndexQuerySet

logger = logging.getLogger(__name__)


class MeiliGeo(TypedDict):
    lat: float | str
    lng: float | str


class _Meili(TypedDict):
    primary_key: str
    index_name: str
    displayed_fields: Iterable[str] | None
    searchable_fields: Iterable[str] | None
    filterable_fields: Iterable[str] | None
    sortable_fields: Iterable[str] | None
    supports_geo: bool
    tasks: list[Task | TaskInfo]
    include_pk_in_search: bool


class IndexMixin(models.Model):
    meilisearch: IndexQuerySet
    _meilisearch: _Meili

    objects: models.Manager

    class Meta:
        abstract = True

    class MeiliMeta:
        displayed_fields: Iterable[str] | None = None
        searchable_fields: Iterable[str] | None = None
        filterable_fields: Iterable[str] | None = None
        sortable_fields: Iterable[str] | None = None
        ranking_rules: Iterable[str] | None = None
        stop_words: Iterable[str] | None = None
        synonyms: dict[str, list[str]] | None = None
        distinct_attribute: str | None = None
        typo_tolerance: dict[str, Any] | None = None
        faceting: dict[str, Any] | None = None
        pagination: dict[str, Any] | None = None
        supports_geo: bool = False
        index_name: str | None = None
        primary_key: str = "pk"
        include_pk_in_search: bool = False

    def __init_subclass__(cls) -> None:
        index_name = getattr(cls.MeiliMeta, "index_name", cls.__name__)
        primary_key = getattr(cls.MeiliMeta, "primary_key", "pk")
        displayed_fields = getattr(cls.MeiliMeta, "displayed_fields", None)
        searchable_fields = getattr(cls.MeiliMeta, "searchable_fields", None)
        filterable_fields = getattr(cls.MeiliMeta, "filterable_fields", None)
        sortable_fields = getattr(cls.MeiliMeta, "sortable_fields", None)
        ranking_rules = getattr(cls.MeiliMeta, "ranking_rules", None)
        stop_words = getattr(cls.MeiliMeta, "stop_words", None)
        synonyms = getattr(cls.MeiliMeta, "synonyms", None)
        distinct_attribute = getattr(cls.MeiliMeta, "distinct_attribute", None)
        typo_tolerance = getattr(cls.MeiliMeta, "typo_tolerance", None)
        faceting = getattr(cls.MeiliMeta, "faceting", None)
        pagination = getattr(cls.MeiliMeta, "pagination", None)
        supports_geo = getattr(cls.MeiliMeta, "supports_geo", False)
        include_pk_in_search = getattr(
            cls.MeiliMeta, "include_pk_in_search", False
        )

        if supports_geo:
            filterable_fields = ("_geo",) + (filterable_fields or ())
            sortable_fields = ("_geo",) + (sortable_fields or ())

        if settings.MEILISEARCH.get("OFFLINE", False):
            cls._meilisearch = _Meili(
                primary_key=primary_key,
                index_name=index_name,
                displayed_fields=displayed_fields,
                searchable_fields=searchable_fields,
                filterable_fields=filterable_fields,
                sortable_fields=sortable_fields,
                supports_geo=supports_geo,
                include_pk_in_search=include_pk_in_search,
                tasks=[],
            )
        else:
            index_settings = MeiliIndexSettings(
                displayed_fields=displayed_fields,
                searchable_fields=searchable_fields,
                filterable_fields=filterable_fields,
                sortable_fields=sortable_fields,
                ranking_rules=ranking_rules,
                stop_words=stop_words,
                synonyms=synonyms,
                distinct_attribute=distinct_attribute,
                typo_tolerance=typo_tolerance,
                faceting=faceting,
                pagination=pagination,
            )

            try:
                _client.create_index(index_name, primary_key).with_settings(
                    index_name=index_name,
                    index_settings=index_settings,
                )
            except Exception as e:
                logger.error(f"Failed to create index {index_name}: {e}")
                pass

        cls._meilisearch = _Meili(
            primary_key=primary_key,
            index_name=index_name,
            displayed_fields=displayed_fields,
            searchable_fields=searchable_fields,
            filterable_fields=filterable_fields,
            sortable_fields=sortable_fields,
            supports_geo=supports_geo,
            include_pk_in_search=include_pk_in_search,
            tasks=[task for task in _client.tasks],
        )
        _client.flush_tasks()

        cls.meilisearch = IndexQuerySet(cls)

    def meili_filter(self) -> bool:
        return True

    @classmethod
    def get_additional_meili_fields(cls):
        return {}

    def meili_serialize(self):
        serialized_model = loads(
            serialize(
                "json",
                [self],
                use_natural_foreign_keys=True,
                use_natural_primary_keys=True,
            )
        )[0]

        data = serialized_model["fields"]

        additional_fields = self.get_additional_meili_fields()
        for field_name, value_getter in additional_fields.items():
            try:
                data[field_name] = value_getter(self)
            except AttributeError:
                data[field_name] = None

        if getattr(self.MeiliMeta, "include_pk_in_search", False):
            field = self._meta.get_field(self.MeiliMeta.primary_key)
            if isinstance(field, Field):
                data[self.MeiliMeta.primary_key] = field.value_to_string(self)
            else:
                data[self.MeiliMeta.primary_key] = str(
                    getattr(self, self.MeiliMeta.primary_key)
                )

        return data

    def meili_geo(self) -> MeiliGeo:
        raise ValueError("Model does not support geolocation")



import logging

from asgiref.sync import async_to_sync, sync_to_async
from django.conf import settings
from django.contrib.auth import get_user_model
from django.db.models.signals import m2m_changed
from django.dispatch import receiver

from blog.models import BlogComment
from notification.enum import NotificationKindEnum
from notification.models.notification import Notification
from notification.models.user import NotificationUser

logger = logging.getLogger(__name__)

User = get_user_model()

languages = [
    lang["code"] for lang in settings.PARLER_LANGUAGES[settings.SITE_ID]
]


@receiver(m2m_changed, sender=BlogComment.likes.through)
def notify_comment_liked_receiver(
    sender, instance, action, reverse, pk_set, **kwargs
):
    if action == "post_add" and not reverse:
        async_to_sync(notify_comment_liked)(
            sender=sender,
            instance=instance,
            action=action,
            reverse=reverse,
            pk_set=pk_set,
            **kwargs,
        )


async def notify_comment_liked(
    sender, instance, action, reverse, pk_set, **kwargs
):
    try:
        if not pk_set:
            return

        user = await sync_to_async(getattr)(instance, "user", None)
        if not user:
            return

        post = await sync_to_async(getattr)(instance, "post", None)
        if not post:
            return

        blog_post_url = f"{settings.NUXT_BASE_URL}/blog/post/{post.id}/{post.slug}#blog-post-comments"

        for user_id in pk_set:
            if user.id == user_id:
                continue

            try:
                liker_user = await User.objects.aget(id=user_id)
                notification = await Notification.objects.acreate(
                    kind=NotificationKindEnum.INFO,
                    link=blog_post_url,
                )

                for language in languages:
                    await sync_to_async(notification.set_current_language)(
                        language
                    )

                    title = ""
                    message = ""

                    if language == "en":
                        title = f"<a href='{blog_post_url}'>Comment</a> Liked!"
                        message = (
                            f"Your comment was liked by "
                            f"{liker_user.username or liker_user.email}."
                        )
                    elif language == "el":
                        title = f"Το <a href='{blog_post_url}'>σχόλιο</a> σου πήρε like!"
                        message = (
                            f"Το σχόλιο σου άρεσε στον χρήστη "
                            f"{liker_user.username or liker_user.email}."
                        )

                    if title and message:
                        await sync_to_async(setattr)(
                            notification, "title", title
                        )
                        await sync_to_async(setattr)(
                            notification, "message", message
                        )
                        await notification.asave()

                await NotificationUser.objects.acreate(
                    user=user, notification=notification
                )

            except User.DoesNotExist:
                logger.error(f"User {user_id} does not exist")
                continue

            except Exception as e:
                logger.error(f"Error sending notification: {e}")
                continue

    except Exception as e:
        logger.error(f"Error sending notification: {e}")
        pass



from django.db import models
from django.test import TestCase

from contact.models import Contact


class TestContactManager(TestCase):
    def setUp(self):
        Contact.objects.create(
            name="John Doe",
            email="john@example.com",
            message="Message from example.com domain",
        )

        Contact.objects.create(
            name="Jane Smith",
            email="jane@example.com",
            message="Another message from example.com",
        )

        Contact.objects.create(
            name="Bob Wilson",
            email="bob@gmail.com",
            message="Message from gmail.com domain",
        )

        Contact.objects.create(
            name="Alice Brown",
            email="alice@company.org",
            message="Message from company.org domain",
        )

        Contact.objects.create(
            name="Charlie Davis",
            email="charlie@test.example.com",
            message="Message from subdomain",
        )

    def test_by_email_domain_queryset_method(self):
        example_contacts = Contact.objects.filter().by_email_domain(
            "example.com"
        )
        assert example_contacts.count() == 2

        gmail_contacts = Contact.objects.filter().by_email_domain("gmail.com")
        assert gmail_contacts.count() == 1

        org_contacts = Contact.objects.filter().by_email_domain("company.org")
        assert org_contacts.count() == 1

    def test_by_email_domain_manager_method(self):
        example_contacts = Contact.objects.by_email_domain("example.com")
        assert example_contacts.count() == 2

        gmail_contacts = Contact.objects.by_email_domain("gmail.com")
        assert gmail_contacts.count() == 1

        org_contacts = Contact.objects.by_email_domain("company.org")
        assert org_contacts.count() == 1

    def test_by_email_domain_case_insensitive(self):
        upper_case = Contact.objects.by_email_domain("EXAMPLE.COM")
        lower_case = Contact.objects.by_email_domain("example.com")
        mixed_case = Contact.objects.by_email_domain("Example.Com")

        assert (
            upper_case.count() == lower_case.count() == mixed_case.count() == 2
        )

    def test_by_email_domain_empty_domain(self):
        empty_contacts = Contact.objects.by_email_domain("")
        assert empty_contacts.count() == 0

    def test_by_email_domain_nonexistent_domain(self):
        nonexistent_contacts = Contact.objects.by_email_domain(
            "nonexistent.com"
        )
        assert nonexistent_contacts.count() == 0

    def test_by_email_domain_subdomain_matching(self):
        example_contacts = Contact.objects.by_email_domain("example.com")

        emails = [contact.email for contact in example_contacts]
        assert "john@example.com" in emails
        assert "jane@example.com" in emails
        assert "charlie@test.example.com" not in emails

        subdomain_contacts = Contact.objects.by_email_domain("test.example.com")
        subdomain_emails = [contact.email for contact in subdomain_contacts]
        assert "charlie@test.example.com" in subdomain_emails

    def test_by_email_domain_partial_domain(self):
        partial_contacts = Contact.objects.by_email_domain("example")
        assert partial_contacts.count() == 0

        tld_contacts = Contact.objects.by_email_domain("com")
        assert tld_contacts.count() == 0

    def test_by_email_domain_special_characters(self):
        Contact.objects.create(
            name="Test User",
            email="test@my-domain.co.uk",
            message="Test message",
        )

        special_contacts = Contact.objects.by_email_domain("my-domain.co.uk")
        assert special_contacts.count() == 1

    def test_by_email_domain_chaining(self):
        example_contacts = Contact.objects.by_email_domain(
            "example.com"
        ).filter(name__startswith="J")
        assert example_contacts.count() == 2

        ordered_contacts = Contact.objects.by_email_domain(
            "example.com"
        ).order_by("name")
        names = [contact.name for contact in ordered_contacts]
        assert names == ["Jane Smith", "John Doe"]

    def test_by_email_domain_with_exclude(self):
        non_gmail_contacts = Contact.objects.exclude(
            email__iendswith="@gmail.com"
        ).by_email_domain("example.com")

        assert non_gmail_contacts.count() == 2

    def test_manager_returns_correct_queryset_type(self):
        queryset = Contact.objects.get_queryset()
        assert hasattr(queryset, "by_email_domain")

        domain_queryset = Contact.objects.by_email_domain("example.com")
        assert hasattr(domain_queryset, "by_email_domain")

    def test_by_email_domain_with_annotations(self):
        annotated_contacts = Contact.objects.annotate(
            name_length=models.Count("name")
        ).by_email_domain("example.com")

        assert annotated_contacts.count() == 2

    def test_by_email_domain_with_values(self):
        domain_emails = Contact.objects.by_email_domain(
            "example.com"
        ).values_list("email", flat=True)

        emails = list(domain_emails)
        assert len(emails) == 2
        assert all("example.com" in email for email in emails)

    def test_by_email_domain_unicode_domain(self):
        Contact.objects.create(
            name="Unicode User",
            email="test@müller.de",
            message="Unicode domain test",
        )

        unicode_contacts = Contact.objects.by_email_domain("müller.de")
        assert unicode_contacts.count() == 1

    def test_by_email_domain_with_multiple_at_signs(self):
        Contact.objects.create(
            name="Malformed User",
            email="test@@malformed.com",
            message="Malformed email test",
        )

        malformed_contacts = Contact.objects.by_email_domain("malformed.com")
        assert malformed_contacts.count() == 1

    def test_get_queryset_method(self):
        manager = Contact.objects
        queryset = manager.get_queryset()

        assert hasattr(queryset, "by_email_domain")
        assert queryset.model == Contact

    def test_manager_inheritance(self):
        manager = Contact.objects
        assert isinstance(manager, models.Manager)

    def test_by_email_domain_performance(self):
        with self.assertNumQueries(1):
            list(Contact.objects.by_email_domain("example.com"))

    def test_by_email_domain_empty_result_set(self):
        empty_qs = Contact.objects.by_email_domain("nowhere.com")

        assert empty_qs.count() == 0
        assert list(empty_qs) == []

        chained = empty_qs.filter(name="Test")
        assert chained.count() == 0



# Generated by Django 5.0.3 on 2024-03-09 14:54
import django.contrib.postgres.indexes
from django.db import migrations
from django.db import models


class Migration(migrations.Migration):
    dependencies = [
        ("product", "0003_alter_product_weight"),
        ("vat", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="product",
            name="search_document",
            field=models.TextField(blank=True, default=""),
        ),
        migrations.AddField(
            model_name="product",
            name="search_index_dirty",
            field=models.BooleanField(db_index=True, default=False),
        ),
        migrations.AddIndex(
            model_name="product",
            index=django.contrib.postgres.indexes.GinIndex(
                fields=["search_document"],
                name="product_search_gin",
                opclasses=["gin_trgm_ops"],
            ),
        ),
    ]



from django.conf import settings
from django.contrib.auth import get_user_model
from django.urls import reverse
from rest_framework import status
from rest_framework.test import APITestCase

from blog.factories.author import BlogAuthorFactory
from blog.factories.category import BlogCategoryFactory
from blog.factories.post import BlogPostFactory
from blog.factories.tag import BlogTagFactory
from blog.models.post import BlogPost
from core.utils.testing import TestURLFixerMixin
from user.factories.account import UserAccountFactory

languages = [
    lang["code"] for lang in settings.PARLER_LANGUAGES[settings.SITE_ID]
]
default_language = settings.PARLER_DEFAULT_LANGUAGE_CODE
User = get_user_model()


class BlogPostViewSetTestCase(TestURLFixerMixin, APITestCase):
    @classmethod
    def setUpTestData(cls):
        cls.user = UserAccountFactory()
        cls.author = BlogAuthorFactory(user=cls.user)
        cls.category = BlogCategoryFactory()
        cls.tag1 = BlogTagFactory(active=True)
        cls.tag2 = BlogTagFactory(active=True)
        cls.post = BlogPostFactory(
            author=cls.author,
            category=cls.category,
            featured=True,
        )
        cls.post.tags.set([cls.tag1, cls.tag2])

    def get_post_list_url(self):
        return reverse("blog-post-list")

    def get_post_detail_url(self, pk):
        return reverse("blog-post-detail", args=[pk])

    def test_list(self):
        url = self.get_post_list_url()
        response = self.client.get(url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertIn("results", response.data)
        self.assertIn("count", response.data)
        if "links" in response.data:
            self.assertIn("next", response.data["links"])
            self.assertIn("previous", response.data["links"])
        else:
            self.assertIn("next", response.data)
            self.assertIn("previous", response.data)

        self.assertGreaterEqual(len(response.data["results"]), 1)

        first_result = response.data["results"][0]
        expected_basic_fields = {
            "id",
            "uuid",
            "slug",
            "likes",
            "translations",
            "author",
            "category",
            "tags",
            "featured",
            "view_count",
            "likes_count",
            "comments_count",
            "tags_count",
            "is_published",
            "published_at",
            "created_at",
            "updated_at",
            "main_image_path",
            "reading_time",
            "content_preview",
        }
        self.assertTrue(
            expected_basic_fields.issubset(set(first_result.keys()))
        )

    def test_list_uses_correct_serializer(self):
        url = self.get_post_list_url()
        response = self.client.get(url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        first_result = response.data["results"][0]
        expected_fields = {
            "id",
            "uuid",
            "slug",
            "likes",
            "translations",
            "author",
            "category",
            "tags",
            "featured",
            "view_count",
            "likes_count",
            "comments_count",
            "tags_count",
            "is_published",
            "published_at",
            "created_at",
            "updated_at",
            "main_image_path",
            "reading_time",
            "content_preview",
        }
        self.assertTrue(expected_fields.issubset(set(first_result.keys())))

    def test_retrieve_valid(self):
        url = self.get_post_detail_url(self.post.id)
        response = self.client.get(url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        expected_fields = {
            "id",
            "uuid",
            "slug",
            "likes",
            "translations",
            "author",
            "category",
            "tags",
            "featured",
            "view_count",
            "likes_count",
            "comments_count",
            "tags_count",
            "is_published",
            "published_at",
            "created_at",
            "updated_at",
            "main_image_path",
            "reading_time",
            "content_preview",
            "user_has_liked",
            "seo_title",
            "seo_description",
            "seo_keywords",
        }
        self.assertTrue(expected_fields.issubset(set(response.data.keys())))

    def test_retrieve_uses_correct_serializer(self):
        url = self.get_post_detail_url(self.post.id)
        response = self.client.get(url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        detail_only_fields = {
            "user_has_liked",
            "seo_title",
            "seo_description",
            "seo_keywords",
        }
        response_fields = set(response.data.keys())

        for field in detail_only_fields:
            self.assertIn(
                field,
                response_fields,
                f"Detail field '{field}' should be present in detail response",
            )

    def test_retrieve_invalid(self):
        invalid_post_id = 9999
        url = self.get_post_detail_url(invalid_post_id)
        response = self.client.get(url)
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_create_valid(self):
        self.client.force_authenticate(user=self.user)
        payload = {
            "slug": "new-test-post",
            "category": self.category.id,
            "tags": [self.tag1.id],
            "author": self.author.id,
            "featured": False,
            "translations": {},
        }

        for language in languages:
            language_code = language
            translation_payload = {
                "title": f"New Test Post Title in {language_code}",
                "subtitle": f"New Test Post Subtitle in {language_code}",
                "body": f"This is a new test post body in {language_code}",
            }
            payload["translations"][language_code] = translation_payload

        url = self.get_post_list_url()
        response = self.client.post(url, payload, format="json")
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

        expected_fields = {
            "id",
            "uuid",
            "slug",
            "likes",
            "translations",
            "author",
            "category",
            "tags",
            "featured",
            "view_count",
            "likes_count",
            "comments_count",
            "tags_count",
            "is_published",
            "published_at",
            "created_at",
            "updated_at",
            "main_image_path",
            "reading_time",
            "content_preview",
            "user_has_liked",
            "seo_title",
            "seo_description",
            "seo_keywords",
        }
        self.assertTrue(expected_fields.issubset(set(response.data.keys())))

    def test_create_invalid(self):
        self.client.force_authenticate(user=self.user)
        payload = {
            "author": "invalid_author_id",
            "translations": {
                "invalid_lang_code": {
                    "title": "Translation for invalid language code",
                },
            },
        }

        url = self.get_post_list_url()
        response = self.client.post(url, payload, format="json")
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    def test_update_valid(self):
        self.client.force_authenticate(user=self.user)
        payload = {
            "slug": "updated-test-post",
            "category": self.category.id,
            "tags": [self.tag1.id],
            "author": self.author.id,
            "translations": {},
        }

        for language in languages:
            language_code = language
            translation_payload = {
                "title": f"Updated Test Post Title in {language_code}",
                "subtitle": f"Updated Test Post Subtitle in {language_code}",
                "body": f"This is an updated test post body in {language_code}",
            }
            payload["translations"][language_code] = translation_payload

        url = self.get_post_detail_url(self.post.id)
        response = self.client.put(url, payload, format="json")
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        expected_fields = {
            "id",
            "uuid",
            "slug",
            "likes",
            "translations",
            "author",
            "category",
            "tags",
            "featured",
            "view_count",
            "likes_count",
            "comments_count",
            "tags_count",
            "is_published",
            "published_at",
            "created_at",
            "updated_at",
            "main_image_path",
            "reading_time",
            "content_preview",
            "user_has_liked",
            "seo_title",
            "seo_description",
            "seo_keywords",
        }
        self.assertTrue(expected_fields.issubset(set(response.data.keys())))

    def test_update_invalid(self):
        self.client.force_authenticate(user=self.user)
        payload = {
            "author": "invalid_author_id",
            "translations": {
                "invalid_lang_code": {
                    "content": "Translation for invalid language code",
                },
            },
        }

        url = self.get_post_detail_url(self.post.id)
        response = self.client.put(url, payload, format="json")
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    def test_partial_update_valid(self):
        self.client.force_authenticate(user=self.user)
        payload = {
            "translations": {
                default_language: {"title": "Partially Updated Post Title"}
            }
        }

        url = self.get_post_detail_url(self.post.id)
        response = self.client.patch(url, payload, format="json")
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        expected_fields = {
            "id",
            "uuid",
            "slug",
            "likes",
            "translations",
            "author",
            "category",
            "tags",
            "featured",
            "view_count",
            "likes_count",
            "comments_count",
            "tags_count",
            "is_published",
            "published_at",
            "created_at",
            "updated_at",
            "main_image_path",
            "reading_time",
            "content_preview",
            "user_has_liked",
            "seo_title",
            "seo_description",
            "seo_keywords",
        }
        self.assertTrue(expected_fields.issubset(set(response.data.keys())))

    def test_partial_update_invalid(self):
        self.client.force_authenticate(user=self.user)
        payload = {
            "slug": "",
            "translations": {default_language: {"title": "Test Post"}},
        }

        url = self.get_post_detail_url(self.post.id)
        response = self.client.patch(url, payload, format="json")
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    def test_destroy_valid(self):
        self.client.force_authenticate(user=self.user)
        post_to_delete = BlogPostFactory(author=self.author)

        url = self.get_post_detail_url(post_to_delete.id)
        response = self.client.delete(url)
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

        self.assertFalse(BlogPost.objects.filter(id=post_to_delete.id).exists())

    def test_destroy_invalid(self):
        self.client.force_authenticate(user=self.user)
        invalid_post_id = 9999

        url = self.get_post_detail_url(invalid_post_id)
        response = self.client.delete(url)
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_related_posts_response_structure(self):
        url = reverse("blog-post-related_posts", args=[self.post.id])
        response = self.client.get(url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertIsInstance(response.data, list)

    def test_related_posts_when_no_related_posts_exist(self):
        BlogPost.objects.exclude(id=self.post.id).delete()

        url = reverse("blog-post-related_posts", args=[self.post.id])
        response = self.client.get(url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(len(response.data), 0)

    def test_related_posts_default_strategy_fills_limit(self):
        additional_default_posts = BlogPostFactory.create_batch(
            3,
            author=self.author,
            category=self.category,
            num_tags=0,
            num_comments=0,
        )
        for post in additional_default_posts:
            post.tags.set([])
            post.likes.set([])

        url = reverse("blog-post-related_posts", args=[self.post.id])
        response = self.client.get(url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertLessEqual(len(response.data), 8)

    def test_comments_action(self):
        url = reverse("blog-post-comments", args=[self.post.id])
        response = self.client.get(url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertIn("results", response.data)
        self.assertIn("count", response.data)

    def test_trending_action(self):
        url = reverse("blog-post-trending")
        response = self.client.get(url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertIn("results", response.data)
        self.assertIn("count", response.data)

    def test_trending_action_with_days_parameter(self):
        url = reverse("blog-post-trending")
        response = self.client.get(url, {"days": 30})
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertIn("results", response.data)
        self.assertIn("count", response.data)

    def test_popular_action(self):
        url = reverse("blog-post-popular")
        response = self.client.get(url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertIn("results", response.data)
        self.assertIn("count", response.data)

    def test_featured_action(self):
        url = reverse("blog-post-featured")
        response = self.client.get(url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertIn("results", response.data)
        self.assertIn("count", response.data)

        for post_data in response.data["results"]:
            self.assertTrue(post_data["featured"])

    def test_update_likes_action_authenticated(self):
        self.client.force_authenticate(user=self.user)
        url = reverse("blog-post-update_likes", args=[self.post.id])
        response = self.client.post(url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_update_likes_action_unauthenticated(self):
        url = reverse("blog-post-update_likes", args=[self.post.id])
        response = self.client.post(url)
        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)

    def test_update_view_count_action(self):
        original_view_count = self.post.view_count
        url = reverse("blog-post-update_view_count", args=[self.post.id])
        response = self.client.post(url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.post.refresh_from_db()
        self.assertEqual(self.post.view_count, original_view_count + 1)

    def test_liked_posts_action_authenticated(self):
        self.client.force_authenticate(user=self.user)
        payload = {"post_ids": [self.post.id]}

        url = reverse("blog-post-liked_posts")
        response = self.client.post(url, payload, format="json")
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertIn("post_ids", response.data)
        self.assertIsInstance(response.data["post_ids"], list)

    def test_liked_posts_action_invalid_data(self):
        self.client.force_authenticate(user=self.user)
        payload = {"invalid_field": "test"}

        url = reverse("blog-post-liked_posts")
        response = self.client.post(url, payload, format="json")
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    def test_filtering_functionality(self):
        url = self.get_post_list_url()
        response = self.client.get(url, {"featured": "true"})
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        for post_data in response.data["results"]:
            self.assertTrue(post_data["featured"])

    def test_ordering_functionality(self):
        url = self.get_post_list_url()
        response = self.client.get(url, {"ordering": "view_count"})
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertIn("results", response.data)

    def test_search_functionality(self):
        url = self.get_post_list_url()
        response = self.client.get(url, {"search": "test"})
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertIn("results", response.data)



import factory
from django.apps import apps
from django.conf import settings

from blog.models.tag import BlogTag

available_languages = [
    lang["code"] for lang in settings.PARLER_LANGUAGES[settings.SITE_ID]
]


class BlogTagTranslationFactory(factory.django.DjangoModelFactory):
    language_code = factory.Iterator(available_languages)
    name = factory.Faker("text", max_nb_chars=45)
    master = factory.SubFactory("blog.factories.tag.BlogTagFactory")

    class Meta:
        model = apps.get_model("blog", "BlogTagTranslation")
        django_get_or_create = ("language_code", "master")


class BlogTagFactory(factory.django.DjangoModelFactory):
    active = factory.Faker("boolean", chance_of_getting_true=90)

    class Meta:
        model = BlogTag
        skip_postgeneration_save = True

    @factory.post_generation
    def translations(self, create, extracted, **kwargs):
        if not create:
            return

        if extracted is not None:
            self.translations.all().delete()

        if extracted is None and not self.translations.exists():
            translations = [
                BlogTagTranslationFactory(language_code=lang, master=self)
                for lang in available_languages
            ]
        else:
            translations = extracted or []

        for translation in translations:
            translation.master = self
            translation.save()



# Generated by Django 5.0.4 on 2024-04-11 17:27
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("country", "0003_alter_country_options"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="country",
            options={
                "ordering": ["sort_order"],
                "verbose_name": "Country",
                "verbose_name_plural": "Countries",
            },
        ),
    ]



from datetime import timedelta

from django.contrib.postgres.indexes import BTreeIndex
from django.db import models
from django.db.models.functions import Now
from django.utils import timezone
from django.utils.translation import gettext_lazy as _
from django_stubs_ext.db.models import TypedModelMeta
from parler.models import TranslatableModel, TranslatedFields

from core.models import TimeStampMixinModel, UUIDModel
from notification.enum import (
    NotificationCategoryEnum,
    NotificationKindEnum,
    NotificationPriorityEnum,
)
from notification.managers import NotificationManager

EXPIRATION_DAYS = 6 * 30


class Notification(TranslatableModel, TimeStampMixinModel, UUIDModel):
    link = models.URLField(_("Link"), blank=True, default="")
    kind = models.CharField(
        _("Kind"),
        max_length=250,
        choices=NotificationKindEnum,
        default=NotificationKindEnum.INFO,
    )
    category = models.CharField(
        _("Category"),
        max_length=50,
        choices=NotificationCategoryEnum,
        default=NotificationCategoryEnum.SYSTEM,
    )
    priority = models.CharField(
        _("Priority"),
        max_length=20,
        choices=NotificationPriorityEnum,
        default=NotificationPriorityEnum.NORMAL,
    )
    notification_type = models.CharField(
        _("Notification Type"),
        max_length=100,
        blank=True,
        help_text=_("Specific type of notification for categorization"),
    )
    expiry_date = models.DateTimeField(
        _("Expiry Date"),
        null=True,
        blank=True,
        db_default=Now() + timedelta(days=EXPIRATION_DAYS),
    )
    translations = TranslatedFields(
        title=models.CharField(_("Title"), max_length=250),
        message=models.TextField(_("Message")),
    )

    objects: NotificationManager = NotificationManager()

    def __str__(self):
        message_snippet = (
            self.safe_translation_getter("title", any_language=True)[:50]
            + "..."
        )
        return f"{self.get_kind_display()}: {message_snippet}"

    class Meta(TypedModelMeta):
        verbose_name = _("Notification")
        verbose_name_plural = _("Notifications")
        ordering = ["-created_at"]
        indexes = [
            *TimeStampMixinModel.Meta.indexes,
            BTreeIndex(fields=["kind"], name="notification_kind_ix"),
            BTreeIndex(fields=["category"], name="notification_category_ix"),
            BTreeIndex(fields=["priority"], name="notification_priority_ix"),
            BTreeIndex(
                fields=["notification_type"], name="notification_type_ix"
            ),
            BTreeIndex(
                fields=["expiry_date"], name="notification_expiry_date_ix"
            ),
            BTreeIndex(
                fields=["category", "priority"], name="notification_cat_pri_ix"
            ),
        ]

    def is_expired(self):
        return self.expiry_date and timezone.now() > self.expiry_date



from datetime import timedelta
import datetime

from django.contrib import admin
from django.db.models.functions import Length
from django.utils import timezone
from django.utils.html import conditional_escape
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _
from unfold.admin import ModelAdmin
from unfold.contrib.filters.admin import (
    RangeDateTimeFilter,
)

from contact.models import Contact
from core.admin import ExportModelAdmin


class MessageLengthFilter(admin.SimpleListFilter):
    title = _("Message Length")
    parameter_name = "message_length"

    def lookups(self, request, model_admin):
        return [
            ("short", _("Short (<100 chars)")),
            ("medium", _("Medium (100-500 chars)")),
            ("long", _("Long (>500 chars)")),
        ]

    def queryset(self, request, queryset):
        if self.value() == "short":
            return queryset.annotate(msg_len=Length("message")).filter(
                msg_len__lt=100
            )
        elif self.value() == "medium":
            return queryset.annotate(msg_len=Length("message")).filter(
                msg_len__range=(100, 500)
            )
        elif self.value() == "long":
            return queryset.annotate(msg_len=Length("message")).filter(
                msg_len__gt=500
            )
        return queryset


class RecentContactFilter(admin.SimpleListFilter):
    title = _("Contact Period")
    parameter_name = "contact_period"

    def lookups(self, request, model_admin):
        return [
            ("today", _("Today")),
            ("week", _("This Week")),
            ("month", _("This Month")),
            ("quarter", _("This Quarter")),
        ]

    def queryset(self, request, queryset):
        now = timezone.now()
        if self.value() == "today":
            start_date = now.replace(hour=0, minute=0, second=0, microsecond=0)
            return queryset.filter(created_at__gte=start_date)
        elif self.value() == "week":
            return queryset.filter(created_at__gte=now - timedelta(days=7))
        elif self.value() == "month":
            return queryset.filter(created_at__gte=now - timedelta(days=30))
        elif self.value() == "quarter":
            return queryset.filter(created_at__gte=now - timedelta(days=90))
        return queryset


@admin.register(Contact)
class ContactAdmin(ExportModelAdmin, ModelAdmin):
    compressed_fields = True
    warn_unsaved_form = True
    list_fullwidth = True
    list_filter_submit = True
    list_filter_sheet = True

    list_display = [
        "contact_info",
        "message_preview",
        "message_stats",
        "contact_timing",
        "priority_badge",
    ]
    list_filter = [
        RecentContactFilter,
        MessageLengthFilter,
        ("created_at", RangeDateTimeFilter),
        ("updated_at", RangeDateTimeFilter),
    ]
    search_fields = ["name", "email", "message"]
    readonly_fields = (
        "id",
        "uuid",
        "created_at",
        "updated_at",
        "contact_analytics",
        "message_analytics",
        "timing_info",
    )
    list_per_page = 25
    date_hierarchy = "created_at"

    fieldsets = (
        (
            _("Contact Information"),
            {
                "fields": ("name", "email"),
                "classes": ("wide",),
            },
        ),
        (
            _("Message"),
            {
                "fields": ("message",),
                "classes": ("wide",),
            },
        ),
        (
            _("Analytics"),
            {
                "fields": ("contact_analytics", "message_analytics"),
                "classes": ("collapse",),
            },
        ),
        (
            _("Timing Information"),
            {
                "fields": ("timing_info",),
                "classes": ("collapse",),
            },
        ),
        (
            _("System Information"),
            {
                "fields": ("id", "uuid", "created_at", "updated_at"),
                "classes": ("collapse",),
            },
        ),
    )

    def get_queryset(self, request):
        return super().get_queryset(request)

    def get_ordering(self, request):
        return ["-created_at", "name"]

    def contact_info(self, obj):
        name = obj.name[:30] + ("..." if len(obj.name) > 30 else "")
        is_suspicious = (
            not obj.email
            or "@" not in obj.email
            or "." not in obj.email.split("@")[-1]
        )
        email_display = obj.email or "(no email)"

        safe_name = conditional_escape(name)
        safe_email = conditional_escape(email_display)
        safe_email_class = conditional_escape(
            "text-red-600 dark:text-red-400"
            if is_suspicious
            else "text-base-600 dark:text-base-400"
        )
        safe_id = conditional_escape(str(obj.id))

        html = (
            f'<div class="text-sm">'
            f'<div class="font-medium text-base-900 dark:text-base-100">{safe_name}</div>'
            f'<div class="{safe_email_class}">{safe_email}</div>'
            f'<div class="text-xs text-base-500 dark:text-base-400">ID: {safe_id}</div>'
            f"</div>"
        )
        return mark_safe(html)

    contact_info.short_description = _("Contact")

    def message_preview(self, obj):
        full = obj.message or ""
        preview = full[:100] + ("..." if len(full) > 100 else "")
        preview = preview.replace("\n", " ").replace("\r", " ")
        title_attr = full.replace('"', "&quot;")

        safe_title = conditional_escape(title_attr)
        safe_preview = conditional_escape(preview)

        html = (
            f'<div class="text-sm">'
            f'<div class="text-base-900 dark:text-base-100" title="{safe_title}">{safe_preview}</div>'
            f"</div>"
        )
        return mark_safe(html)

    message_preview.short_description = _("Message")

    def message_stats(self, obj):
        msg = obj.message or ""
        char_count = len(msg)
        word_count = len(msg.split())
        line_count = msg.count("\n") + 1

        # Badges
        if char_count < 100:
            length_badge = mark_safe(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium '
                'bg-green-50 dark:bg-green-900 text-green-700 dark:text-green-200 rounded-full">'
                "📝 Short"
                "</span>"
            )
        elif char_count < 500:
            length_badge = mark_safe(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium '
                'bg-yellow-50 dark:bg-yellow-900 text-yellow-700 dark:text-yellow-200 rounded-full">'
                "📄 Medium"
                "</span>"
            )
        else:
            length_badge = mark_safe(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium '
                'bg-blue-50 dark:bg-blue-900 text-blue-700 dark:text-blue-200 rounded-full">'
                "📋 Long"
                "</span>"
            )

        safe_chars = conditional_escape(str(char_count))
        safe_words = conditional_escape(str(word_count))
        safe_lines = conditional_escape(str(line_count))

        html = (
            f'<div class="text-sm">'
            f'<div class="font-medium text-base-900 dark:text-base-100">{safe_chars} chars</div>'
            f'<div class="text-base-600 dark:text-base-400">{safe_words} words</div>'
            f'<div class="text-base-500 dark:text-base-400">{safe_lines} lines</div>'
            f'<div class="mt-1">{length_badge}</div>'
            f"</div>"
        )
        return mark_safe(html)

    message_stats.short_description = _("Message Stats")

    def contact_timing(self, obj):
        now = timezone.now()
        diff = now - obj.created_at
        date_str = obj.created_at.strftime("%Y-%m-%d")

        if diff < timedelta(hours=1):
            time_badge = mark_safe(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium '
                'bg-red-50 dark:bg-red-900 text-red-700 dark:text-red-200 rounded-full">'
                "🚨 Just Now"
                "</span>"
            )
        elif diff < timedelta(days=1):
            hours_ago = str(int(diff.total_seconds() // 3600))
            safe_hours = conditional_escape(hours_ago)
            time_badge = mark_safe(
                f'<span class="inline-flex items-center px-2 py-1 text-xs font-medium '
                f'bg-orange-50 dark:bg-orange-900 text-orange-700 dark:text-orange-200 rounded-full">'
                f"🕒 {safe_hours}h ago"
                f"</span>"
            )
        elif diff < timedelta(days=7):
            days_ago = str(diff.days)
            safe_days = conditional_escape(days_ago)
            time_badge = mark_safe(
                f'<span class="inline-flex items-center px-2 py-1 text-xs font-medium '
                f'bg-yellow-50 dark:bg-yellow-900 text-yellow-700 dark:text-yellow-200 rounded-full">'
                f"📅 {safe_days}d ago"
                f"</span>"
            )
        else:
            time_badge = mark_safe(
                '<span class="inline-flex items-center px-2 py-1 text-xs font-medium '
                'bg-gray-50 dark:bg-gray-900 text-base-700 dark:text-base-200 rounded-full">'
                "📆 Old"
                "</span>"
            )

        safe_date = conditional_escape(date_str)
        html = (
            f'<div class="text-sm">'
            f'<div class="font-medium text-base-900 dark:text-base-100">{safe_date}</div>'
            f'<div class="mt-1">{time_badge}</div>'
            f"</div>"
        )
        return mark_safe(html)

    contact_timing.short_description = _("Timing")

    def priority_badge(self, obj):
        now = timezone.now()
        diff = now - obj.created_at
        length = len(obj.message or "")

        if diff < timedelta(hours=2):
            key = "urgent"
        elif length > 500:
            key = "high"
        elif diff < timedelta(days=1):
            key = "medium"
        else:
            key = "low"

        cfg = {
            "urgent": {
                "bg": "bg-red-50 dark:bg-red-900",
                "text": "text-red-700 dark:text-red-200",
                "icon": "🚨",
                "label": "Urgent",
            },
            "high": {
                "bg": "bg-orange-50 dark:bg-orange-900",
                "text": "text-orange-700 dark:text-orange-200",
                "icon": "⚡",
                "label": "High",
            },
            "medium": {
                "bg": "bg-yellow-50 dark:bg-yellow-900",
                "text": "text-yellow-700 dark:text-yellow-200",
                "icon": "⚠️",
                "label": "Medium",
            },
            "low": {
                "bg": "bg-green-50 dark:bg-green-900",
                "text": "text-green-700 dark:text-green-200",
                "icon": "✅",
                "label": "Low",
            },
        }[key]

        safe_bg = conditional_escape(cfg["bg"])
        safe_text = conditional_escape(cfg["text"])
        safe_icon = conditional_escape(cfg["icon"])
        safe_label = conditional_escape(cfg["label"])

        html = (
            f'<span class="inline-flex items-center justify-center px-2 py-1 text-xs font-medium '
            f'{safe_bg} {safe_text} rounded-full gap-1">'
            f"<span>{safe_icon}</span><span>{safe_label}</span>"
            f"</span>"
        )
        return mark_safe(html)

    priority_badge.short_description = _("Priority")

    def contact_analytics(self, obj):
        now = timezone.now()
        age = now - obj.created_at
        words = len((obj.message or "").split())
        score_map = {"low": 25, "medium": 50, "high": 75, "urgent": 100}

        # reuse priority logic
        if age < timedelta(hours=2):
            prio = "urgent"
        elif len(obj.message or "") > 500:
            prio = "high"
        elif age < timedelta(days=1):
            prio = "medium"
        else:
            prio = "low"
        prio_score = score_map[prio]
        reading_time = max(1, words // 200)

        safe_days = conditional_escape(str(age.days))
        safe_hours = conditional_escape(str(age.seconds // 3600))
        safe_status = conditional_escape(
            "Pending" if age < timedelta(days=1) else "Delayed"
        )
        safe_domain = conditional_escape(
            obj.email.split("@")[-1] if "@" in obj.email else "Invalid"
        )
        safe_dayname = conditional_escape(obj.created_at.strftime("%A"))
        safe_time = conditional_escape(obj.created_at.strftime("%H:%M"))
        safe_reading = conditional_escape(str(reading_time))
        safe_score = conditional_escape(str(prio_score))

        html = (
            f'<div class="text-sm"><div class="grid grid-cols-2 gap-2">'
            f"<div><strong>Contact Age:</strong></div><div>{safe_days}d {safe_hours}h</div>"
            f"<div><strong>Response Time:</strong></div><div>{safe_status}</div>"
            f"<div><strong>Email Domain:</strong></div><div>{safe_domain}</div>"
            f"<div><strong>Contact Day:</strong></div><div>{safe_dayname}</div>"
            f"<div><strong>Contact Time:</strong></div><div>{safe_time}</div>"
            f"<div><strong>Reading Time:</strong></div><div>{safe_reading}min</div>"
            f'<div><strong>Priority Score:</strong></div><div class="font-medium">{safe_score}</div>'
            f"</div></div>"
        )
        return mark_safe(html)

    contact_analytics.short_description = _("Contact Analytics")

    def message_analytics(self, obj):
        msg = obj.message or ""
        chars = len(msg)
        words = len(msg.split())
        sentences = msg.count(".") + msg.count("!") + msg.count("?")
        paras = msg.count("\n\n") + 1
        urgent_terms = [
            "urgent",
            "emergency",
            "asap",
            "immediately",
            "help",
            "problem",
            "issue",
            "broken",
        ]
        urgency = sum(1 for w in urgent_terms if w in msg.lower())
        read_sec = 0 if words == 0 else max(1, int((words / 200) * 60))
        sentiment = "Urgent" if urgency >= 3 else "Neutral"
        avg_len_val = chars / max(words, 1)
        complexity = "Complex" if avg_len_val > 5 else "Simple"
        language = "English"

        safe_chars = conditional_escape(str(chars))
        safe_words = conditional_escape(str(words))
        safe_sentences = conditional_escape(str(sentences))
        safe_paras = conditional_escape(str(paras))
        safe_urgency = conditional_escape(str(urgency))
        safe_avg_len = conditional_escape(f"{avg_len_val:.1f}")
        safe_read_sec = conditional_escape(str(read_sec))
        safe_sentiment = conditional_escape(sentiment)
        safe_complexity = conditional_escape(complexity)
        safe_language = conditional_escape(language)

        html = (
            f'<div class="text-sm"><div class="grid grid-cols-2 gap-2">'
            f"<div><strong>Characters:</strong></div><div>{safe_chars}</div>"
            f"<div><strong>Words:</strong></div><div>{safe_words}</div>"
            f"<div><strong>Sentences:</strong></div><div>{safe_sentences}</div>"
            f"<div><strong>Paragraphs:</strong></div><div>{safe_paras}</div>"
            f"<div><strong>Urgency Score:</strong></div><div>{safe_urgency}/10</div>"
            f"<div><strong>Avg Word Length:</strong></div><div>{safe_avg_len}</div>"
            f"<div><strong>Reading Time:</strong></div><div>{safe_read_sec} seconds</div>"
            f"<div><strong>Sentiment:</strong></div><div>{safe_sentiment}</div>"
            f"<div><strong>Complexity:</strong></div><div>{safe_complexity}</div>"
            f"<div><strong>Language:</strong></div><div>{safe_language}</div>"
            f"</div></div>"
        )
        return mark_safe(html)

    message_analytics.short_description = _("Message Analytics")

    def timing_info(self, obj):
        now = timezone.now()
        created_age = now - obj.created_at
        updated_age = now - obj.updated_at
        is_business = 9 <= obj.created_at.hour <= 17
        is_weekend = obj.created_at.weekday() >= 5
        season = self._get_season(obj.created_at.month)

        safe_created_dt = conditional_escape(
            obj.created_at.strftime("%Y-%m-%d %H:%M:%S")
        )
        safe_updated_dt = conditional_escape(
            obj.updated_at.strftime("%Y-%m-%d %H:%M:%S")
        )
        safe_cdays = conditional_escape(str(created_age.days))
        safe_chours = conditional_escape(str(created_age.seconds // 3600))
        safe_udays = conditional_escape(str(updated_age.days))
        safe_bhours = conditional_escape("Yes" if is_business else "No")
        safe_weekend = conditional_escape("Yes" if is_weekend else "No")
        safe_season = conditional_escape(season)

        html = (
            f'<div class="text-sm"><div class="grid grid-cols-2 gap-2">'
            f"<div><strong>Created:</strong></div><div>{safe_created_dt}</div>"
            f"<div><strong>Updated:</strong></div><div>{safe_updated_dt}</div>"
            f"<div><strong>Age:</strong></div><div>{safe_cdays}d {safe_chours}h</div>"
            f"<div><strong>Last Modified:</strong></div><div>{safe_udays}d ago</div>"
            f"<div><strong>Business Hours:</strong></div><div>{safe_bhours}</div>"
            f"<div><strong>Weekend Contact:</strong></div><div>{safe_weekend}</div>"
            f"<div><strong>Season:</strong></div><div>{safe_season}</div>"
            f"</div></div>"
        )
        return mark_safe(html)

    timing_info.short_description = _("Timing Information")

    def _get_season(self, month_or_date):
        if isinstance(month_or_date, (datetime.date, datetime.datetime)):
            m = month_or_date.month
        else:
            m = int(month_or_date)
        if m in (12, 1, 2):
            return "Winter"
        if m in (3, 4, 5):
            return "Spring"
        if m in (6, 7, 8):
            return "Summer"
        return "Autumn"



import json
from dataclasses import dataclass
from datetime import timedelta

from django.contrib.auth import get_user_model
from django.core.management.base import BaseCommand
from django.utils import timezone
from knox.models import AuthToken
from knox.settings import knox_settings

User = get_user_model()


@dataclass
class TokenContext:
    token: str
    user: User
    created: str
    digest: str
    expiry_str: str
    expires_in: float | None
    output_path: str


class Command(BaseCommand):
    help = "Generate a Knox token for development"

    def add_arguments(self, parser):
        parser.add_argument(
            "--username", type=str, help="Username for token generation"
        )
        parser.add_argument(
            "--email", type=str, help="Email for token generation"
        )
        parser.add_argument(
            "--output",
            type=str,
            default=".auth-token",
            help="Output file for token (default: .auth-token)",
        )
        parser.add_argument(
            "--format",
            type=str,
            choices=["plain", "json", "env"],
            default="plain",
            help="Output format: plain, json, or env",
        )
        parser.add_argument(
            "--ttl",
            type=int,
            default=24 * 30,
            help="Token TTL in hours; overrides default Knox settings",
        )

    def handle(self, *args, **options):
        try:
            user = self._get_user(options)
        except User.DoesNotExist:
            self.stderr.write(self.style.ERROR("User not found"))
            return
        instance, token = self._create_token(user, options["ttl"])
        expiry_str, _, expires_in = self._compute_expiry(instance)
        ctx = TokenContext(
            token=token,
            user=user,
            created=instance.created.isoformat(),
            digest=instance.digest,
            expiry_str=expiry_str,
            expires_in=expires_in,
            output_path=options["output"],
        )
        self._write_token_file(ctx, options["format"])
        self._print_summary(
            user,
            token,
            expiry_str,
            expires_in,
            ctx.output_path,
            options["format"],
        )

    def _get_user(self, opts):
        if not opts.get("username") and not opts.get("email"):
            raise User.DoesNotExist
        lookup = (
            {"username": opts.get("username")}
            if opts.get("username")
            else {"email": opts.get("email")}
        )
        return User.objects.get(**lookup)

    def _create_token(
        self, user: User, ttl_hours: int | None
    ) -> tuple[AuthToken, str]:
        delta = timedelta(hours=ttl_hours) if ttl_hours else None
        return AuthToken.objects.create(user=user, expiry=delta)

    def _compute_expiry(
        self, instance: AuthToken
    ) -> tuple[str, str | None, float | None]:
        if instance.expiry:
            expiry_time = instance.expiry
        elif knox_settings.TOKEN_TTL:
            expiry_time = instance.created + knox_settings.TOKEN_TTL
        else:
            return "Never", None, None

        local_expiry = timezone.localtime(expiry_time)
        expiry_str = local_expiry.strftime("%Y-%m-%d %H:%M:%S %Z")

        expires_in = (expiry_time - timezone.now()).total_seconds()
        return expiry_str, local_expiry.isoformat(), expires_in

    def _write_token_file(self, ctx: TokenContext, fmt: str) -> None:
        if fmt == "plain":
            with open(ctx.output_path, "w") as fp:
                fp.write(ctx.token)
        elif fmt == "json":
            payload = {
                "token": ctx.token,
                "user": {
                    "id": ctx.user.id,
                    "username": ctx.user.username,
                    "email": ctx.user.email,
                },
                "created": ctx.created,
                "expiry": None if ctx.expiry_str == "Never" else ctx.expiry_str,
                "expires_in_seconds": ctx.expires_in,
                "digest": ctx.digest,
            }
            with open(ctx.output_path, "w") as fp:
                json.dump(payload, fp, indent=2)
        elif fmt == "env":
            with open(ctx.output_path, "w") as fp:
                fp.write(
                    f"# Generated on {timezone.localtime():%Y-%m-%d %H:%M:%S %Z}\n"
                    f"# User: {ctx.user.username} ({ctx.user.email})\n"
                    f"# Expires: {ctx.expiry_str}\n"
                    f"AUTH_TOKEN={ctx.token}\n"
                )

    def _print_summary(
        self,
        user: User,
        token: str,
        expiry_str: str,
        expires_in: float | None,
        out_path: str,
        fmt: str,
    ) -> None:
        self.stdout.write(
            self.style.SUCCESS(f"✅ Token generated for {user.username}")
        )
        self.stdout.write(f"📄 Token saved to: {out_path} (format: {fmt})")
        self.stdout.write(f"🔑 Token: {token[:20]}...")
        if expiry_str == "Never":
            self.stdout.write(self.style.SUCCESS("♾️  Token never expires"))
            return
        self.stdout.write(self.style.WARNING(f"⏱️  Token expires: {expiry_str}"))
        hours = expires_in / 3600 if expires_in else 0
        message = (
            f"⏱️  Expires in: {hours:.1f} hours"
            if hours < 24
            else f"⏱️  Expires in: {hours / 24:.1f} days"
        )
        self.stdout.write(self.style.WARNING(message))
        if fmt == "json":
            self.stdout.write(
                self.style.NOTICE(
                    f"\nTo use in JS/TS:\nconst tokenData = require('./{out_path}').token;"
                )
            )
        elif fmt == "env":
            self.stdout.write(
                self.style.NOTICE(
                    f"\nTo use: source {out_path} or add to your .env file"
                )
            )



import json
from unittest.mock import AsyncMock, MagicMock, patch

from channels.db import database_sync_to_async
from channels.testing import WebsocketCommunicator
from django.contrib.auth import get_user_model
from django.contrib.auth.models import AnonymousUser
from django.test import TransactionTestCase

from asgi import application
from notification.consumers import NotificationConsumer

User = get_user_model()


class TestNotificationConsumer(TransactionTestCase):
    async def test_connect_anonymous_user(self):
        communicator = WebsocketCommunicator(application, "ws/notifications/")
        connected, _ = await communicator.connect()
        assert not connected
        await communicator.disconnect()

    async def test_consumer_directly(self):
        user = await self.create_test_user()

        consumer = NotificationConsumer()
        consumer.scope = {"user": user}
        consumer.channel_layer = MagicMock()
        consumer.channel_layer.group_add = AsyncMock()
        consumer.channel_layer.group_discard = AsyncMock()
        consumer.channel_name = "test_channel"
        consumer.accept = AsyncMock()
        consumer.send = AsyncMock()
        consumer.close = AsyncMock()

        await consumer.connect()

        consumer.channel_layer.group_add.assert_called_with(
            f"user_{user.id}", consumer.channel_name
        )

        consumer.accept.assert_called_once()

        test_message = {"text": "Hello, world!"}
        await consumer.receive(text_data=json.dumps(test_message))

        consumer.send.assert_called_with(
            text_data=json.dumps(
                {
                    "type": "echo.message",
                    "message": test_message,
                    "from": user.username,
                }
            )
        )

        consumer.send.reset_mock()
        await consumer.receive(text_data="not json")

        consumer.send.assert_called_with(
            text_data=json.dumps(
                {"type": "error", "message": "Invalid JSON format"}
            )
        )

        await consumer.disconnect(1000)

        consumer.channel_layer.group_discard.assert_any_call(
            f"user_{user.id}", consumer.channel_name
        )

    @database_sync_to_async
    def create_test_user(self):
        return User.objects.create_user(
            username="testuser", email="test@test.com", password="password123"
        )

    @database_sync_to_async
    def create_staff_user(self):
        user = User.objects.create_user(
            username="staffuser", email="staff@test.com", password="password123"
        )
        user.is_staff = True
        user.save()
        return user

    async def test_staff_user_groups(self):
        staff_user = await self.create_staff_user()

        consumer = NotificationConsumer()
        consumer.scope = {"user": staff_user}
        consumer.channel_layer = MagicMock()
        consumer.channel_layer.group_add = AsyncMock()
        consumer.channel_layer.group_discard = AsyncMock()
        consumer.channel_name = "test_channel"
        consumer.accept = AsyncMock()
        consumer.close = AsyncMock()

        await consumer.connect()

        consumer.channel_layer.group_add.assert_any_call(
            f"user_{staff_user.id}", consumer.channel_name
        )
        consumer.channel_layer.group_add.assert_any_call(
            "admins", consumer.channel_name
        )

        await consumer.disconnect(1000)

        consumer.channel_layer.group_discard.assert_any_call(
            f"user_{staff_user.id}", consumer.channel_name
        )
        consumer.channel_layer.group_discard.assert_any_call(
            "admins", consumer.channel_name
        )

    async def test_send_notification(self):
        consumer = NotificationConsumer()
        consumer.send = AsyncMock()

        notification = {
            "type": "send_notification",
            "message": "Test notification",
            "level": "info",
        }

        await consumer.send_notification(notification)

        consumer.send.assert_called_once_with(
            text_data=json.dumps(notification)
        )

    async def test_connect_error_handling(self):
        consumer = NotificationConsumer()
        consumer.close = AsyncMock()

        consumer.scope = {}
        await consumer.connect()
        consumer.close.assert_called_with(code=4000)

        consumer.close.reset_mock()
        consumer.scope = {"user": "not a user object"}
        await consumer.connect()
        consumer.close.assert_called_with(code=4500)

    @patch("notification.consumers.logger")
    async def test_connect_with_logging_authenticated_user(self, mock_logger):
        user = await self.create_test_user()

        consumer = NotificationConsumer()
        consumer.scope = {"user": user}
        consumer.channel_layer = MagicMock()
        consumer.channel_layer.group_add = AsyncMock()
        consumer.channel_name = "test_channel"
        consumer.accept = AsyncMock()

        await consumer.connect()

        mock_logger.debug.assert_any_call("NotificationConsumer connect called")
        mock_logger.debug.assert_any_call(f"User from scope: {user.username}")
        mock_logger.debug.assert_any_call(
            f"Authenticated user: {user.username} (ID: {user.id})"
        )
        mock_logger.debug.assert_any_call(
            f"Adding user to group: user_{user.id}"
        )
        mock_logger.debug.assert_any_call("Accepting connection")
        mock_logger.debug.assert_any_call("Connection accepted")

        self.assertEqual(consumer.user, user)
        self.assertEqual(consumer.group_name, f"user_{user.id}")

    @patch("notification.consumers.logger")
    async def test_connect_with_logging_anonymous_user(self, mock_logger):
        anonymous_user = AnonymousUser()

        consumer = NotificationConsumer()
        consumer.scope = {"user": anonymous_user}
        consumer.close = AsyncMock()

        await consumer.connect()

        mock_logger.debug.assert_any_call("NotificationConsumer connect called")
        mock_logger.debug.assert_any_call("User from scope: AnonymousUser")
        mock_logger.warning.assert_called_with(
            "Anonymous user, closing connection"
        )

        consumer.close.assert_called_with(code=4003)

    @patch("notification.consumers.logger")
    async def test_connect_with_logging_staff_user(self, mock_logger):
        staff_user = await self.create_staff_user()

        consumer = NotificationConsumer()
        consumer.scope = {"user": staff_user}
        consumer.channel_layer = MagicMock()
        consumer.channel_layer.group_add = AsyncMock()
        consumer.channel_name = "test_channel"
        consumer.accept = AsyncMock()

        await consumer.connect()

        mock_logger.debug.assert_any_call(
            "User is staff, adding to admins group"
        )

        consumer.channel_layer.group_add.assert_any_call(
            f"user_{staff_user.id}", "test_channel"
        )
        consumer.channel_layer.group_add.assert_any_call(
            "admins", "test_channel"
        )

    @patch("notification.consumers.logger")
    async def test_connect_keyerror_handling(self, mock_logger):
        consumer = NotificationConsumer()
        consumer.scope = {}
        consumer.close = AsyncMock()

        await consumer.connect()

        mock_logger.error.assert_called()
        consumer.close.assert_called_with(code=4000)

    @patch("notification.consumers.logger")
    async def test_connect_exception_handling(self, mock_logger):
        consumer = NotificationConsumer()
        consumer.scope = {"user": "invalid_user_object"}
        consumer.close = AsyncMock()

        await consumer.connect()

        mock_logger.exception.assert_called()
        consumer.close.assert_called_with(code=4500)

    @patch("notification.consumers.logger")
    async def test_disconnect_with_logging_no_user(self, mock_logger):
        consumer = NotificationConsumer()
        consumer.user = None

        await consumer.disconnect(1000)

        mock_logger.debug.assert_any_call(
            "NotificationConsumer disconnect called with code: 1000"
        )
        mock_logger.debug.assert_any_call("No user in disconnect")

    @patch("notification.consumers.logger")
    async def test_disconnect_with_logging_authenticated_user(
        self, mock_logger
    ):
        user = await self.create_test_user()

        consumer = NotificationConsumer()
        consumer.user = user
        consumer.group_name = f"user_{user.id}"
        consumer.channel_layer = MagicMock()
        consumer.channel_layer.group_discard = AsyncMock()
        consumer.channel_name = "test_channel"

        await consumer.disconnect(1000)

        mock_logger.debug.assert_any_call(
            "NotificationConsumer disconnect called with code: 1000"
        )
        mock_logger.debug.assert_any_call(
            f"Removing user from group: user_{user.id}"
        )

        consumer.channel_layer.group_discard.assert_called_with(
            f"user_{user.id}", "test_channel"
        )

    @patch("notification.consumers.logger")
    async def test_disconnect_with_logging_staff_user(self, mock_logger):
        staff_user = await self.create_staff_user()

        consumer = NotificationConsumer()
        consumer.user = staff_user
        consumer.group_name = f"user_{staff_user.id}"
        consumer.channel_layer = MagicMock()
        consumer.channel_layer.group_discard = AsyncMock()
        consumer.channel_name = "test_channel"

        await consumer.disconnect(1000)

        mock_logger.debug.assert_any_call(
            "User is staff, removing from admins group"
        )

        consumer.channel_layer.group_discard.assert_any_call(
            f"user_{staff_user.id}", "test_channel"
        )
        consumer.channel_layer.group_discard.assert_any_call(
            "admins", "test_channel"
        )

    @patch("notification.consumers.logger")
    async def test_disconnect_with_anonymous_user(self, mock_logger):
        consumer = NotificationConsumer()
        consumer.user = AnonymousUser()

        await consumer.disconnect(1000)

        mock_logger.debug.assert_any_call(
            "NotificationConsumer disconnect called with code: 1000"
        )

    @patch("notification.consumers.logger")
    async def test_send_notification_with_logging(self, mock_logger):
        consumer = NotificationConsumer()
        consumer.send = AsyncMock()

        event = {"type": "test", "message": "Test notification"}
        await consumer.send_notification(event)

        mock_logger.debug.assert_called_with(f"Sending notification: {event}")
        consumer.send.assert_called_with(text_data=json.dumps(event))

    @patch("notification.consumers.logger")
    async def test_receive_with_logging_valid_json(self, mock_logger):
        user = await self.create_test_user()

        consumer = NotificationConsumer()
        consumer.user = user
        consumer.send = AsyncMock()

        test_data = {"message": "test"}
        await consumer.receive(text_data=json.dumps(test_data))

        mock_logger.debug.assert_called_with(
            f"Received message: {json.dumps(test_data)}"
        )

        expected_response = {
            "type": "echo.message",
            "message": test_data,
            "from": user.username,
        }
        consumer.send.assert_called_with(
            text_data=json.dumps(expected_response)
        )

    @patch("notification.consumers.logger")
    async def test_receive_with_logging_invalid_json(self, mock_logger):
        consumer = NotificationConsumer()
        consumer.send = AsyncMock()

        invalid_json = "not valid json"
        await consumer.receive(text_data=invalid_json)

        mock_logger.debug.assert_called_with(
            f"Received message: {invalid_json}"
        )
        mock_logger.error.assert_called_with(
            f"Invalid JSON received: {invalid_json}"
        )

        expected_response = {"type": "error", "message": "Invalid JSON format"}
        consumer.send.assert_called_with(
            text_data=json.dumps(expected_response)
        )

    @patch("notification.consumers.logger")
    async def test_receive_with_anonymous_user(self, mock_logger):
        consumer = NotificationConsumer()
        consumer.user = None
        consumer.send = AsyncMock()

        test_data = {"message": "test"}
        await consumer.receive(text_data=json.dumps(test_data))

        expected_response = {
            "type": "echo.message",
            "message": test_data,
            "from": "anonymous",
        }
        consumer.send.assert_called_with(
            text_data=json.dumps(expected_response)
        )

    @patch("notification.consumers.logger")
    async def test_receive_with_anonymous_user_object(self, mock_logger):
        consumer = NotificationConsumer()
        consumer.user = AnonymousUser()
        consumer.send = AsyncMock()

        test_data = {"message": "test"}
        await consumer.receive(text_data=json.dumps(test_data))

        expected_response = {
            "type": "echo.message",
            "message": test_data,
            "from": "anonymous",
        }
        consumer.send.assert_called_with(
            text_data=json.dumps(expected_response)
        )



from django.contrib.auth.validators import UnicodeUsernameValidator
from django.utils.deconstruct import deconstructible
from django.utils.translation import gettext_lazy as _


@deconstructible
class ExtendedUnicodeUsernameValidator(UnicodeUsernameValidator):
    regex = r"^[\w.@+#-]+\Z"
    message = _(
        "Enter a valid username. This value may contain only letters, "
        "numbers, and @/./+/-/_/# characters."
    )



from django.conf import settings
from django.urls import reverse
from rest_framework import status
from rest_framework.test import APITestCase

from core.utils.testing import TestURLFixerMixin
from country.factories import CountryFactory
from country.models import Country
from country.serializers import (
    CountryDetailSerializer,
    CountrySerializer,
)

languages = [
    lang["code"] for lang in settings.PARLER_LANGUAGES[settings.SITE_ID]
]
default_language = settings.PARLER_DEFAULT_LANGUAGE_CODE


class CountryViewSetTestCase(TestURLFixerMixin, APITestCase):
    @classmethod
    def setUpTestData(cls):
        cls.country = CountryFactory(
            alpha_2="GR",
            alpha_3="GRC",
            iso_cc=300,
            phone_code=30,
            num_regions=0,
        )
        cls.country2 = CountryFactory(
            alpha_2="US",
            alpha_3="USA",
            iso_cc=840,
            phone_code=1,
            num_regions=0,
        )
        cls.country3 = CountryFactory(
            alpha_2="FR",
            alpha_3="FRA",
            iso_cc=250,
            phone_code=33,
            num_regions=0,
        )

    def setUp(self):
        self.list_url = reverse("country-list")
        self.detail_url = reverse("country-detail", args=[self.country.pk])

    def test_list_uses_correct_serializer(self):
        response = self.client.get(self.list_url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertIn("results", response.data)
        self.assertIn("count", response.data)

        if response.data["results"]:
            serializer = CountrySerializer(instance=self.country)
            expected_fields = set(serializer.data.keys())
            actual_fields = set(response.data["results"][0].keys())
            self.assertEqual(expected_fields, actual_fields)

    def test_retrieve_uses_correct_serializer(self):
        response = self.client.get(self.detail_url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        serializer = CountryDetailSerializer(instance=self.country)
        expected_fields = set(serializer.data.keys())
        actual_fields = set(response.data.keys())
        self.assertEqual(expected_fields, actual_fields)

    def test_create_uses_correct_serializer(self):
        payload = {
            "alpha_2": "CY",
            "alpha_3": "CYP",
            "iso_cc": 196,
            "phone_code": 997,
            "translations": {default_language: {"name": "Cyprus"}},
        }

        response = self.client.post(self.list_url, payload, format="json")
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

        created_country = Country.objects.get(alpha_2="CY")
        serializer = CountryDetailSerializer(instance=created_country)
        expected_fields = set(serializer.data.keys())
        actual_fields = set(response.data.keys())
        self.assertEqual(expected_fields, actual_fields)

    def test_update_uses_correct_serializer(self):
        payload = {
            "alpha_2": "GR",
            "alpha_3": "GRC",
            "iso_cc": 301,
            "phone_code": 30,
            "translations": {default_language: {"name": "Updated Greece"}},
        }

        response = self.client.put(self.detail_url, payload, format="json")
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        serializer = CountryDetailSerializer(instance=self.country)
        expected_fields = set(serializer.data.keys())
        actual_fields = set(response.data.keys())
        self.assertEqual(expected_fields, actual_fields)

    def test_list_countries(self):
        response = self.client.get(self.list_url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertIn("results", response.data)
        self.assertIn("count", response.data)

        self.assertGreaterEqual(response.data["count"], 3)

    def test_retrieve_country(self):
        response = self.client.get(self.detail_url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertEqual(response.data["alpha_2"], self.country.alpha_2)
        self.assertEqual(response.data["alpha_3"], self.country.alpha_3)
        self.assertEqual(response.data["iso_cc"], self.country.iso_cc)
        self.assertIn("translations", response.data)

    def test_create_country(self):
        initial_count = Country.objects.count()
        payload = {
            "alpha_2": "CA",
            "alpha_3": "CAN",
            "iso_cc": 124,
            "phone_code": 999,
            "translations": {default_language: {"name": "Canada"}},
        }

        response = self.client.post(self.list_url, payload, format="json")
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        self.assertEqual(Country.objects.count(), initial_count + 1)

        country = Country.objects.get(alpha_2="CA")
        self.assertEqual(country.alpha_3, "CAN")
        self.assertEqual(country.iso_cc, 124)

    def test_update_country(self):
        payload = {
            "alpha_2": "GR",
            "alpha_3": "GRC",
            "iso_cc": 302,
            "phone_code": 30,
            "translations": {default_language: {"name": "Hellenic Republic"}},
        }

        response = self.client.put(self.detail_url, payload, format="json")
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.country.refresh_from_db()
        self.assertEqual(self.country.iso_cc, 302)

    def test_partial_update_country(self):
        payload = {"iso_cc": 303}

        response = self.client.patch(self.detail_url, payload, format="json")
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.country.refresh_from_db()
        self.assertEqual(self.country.iso_cc, 303)

    def test_delete_country(self):
        country_id = self.country.pk
        response = self.client.delete(self.detail_url)
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)
        self.assertFalse(Country.objects.filter(pk=country_id).exists())

    def test_filter_by_alpha_2(self):
        response = self.client.get(self.list_url, {"alpha_2": "GR"})
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(len(response.data["results"]), 1)
        self.assertEqual(response.data["results"][0]["alpha_2"], "GR")

    def test_filter_by_alpha_3(self):
        response = self.client.get(self.list_url, {"alpha_3": "USA"})
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(len(response.data["results"]), 1)
        self.assertEqual(response.data["results"][0]["alpha_3"], "USA")

    def test_filter_by_iso_cc(self):
        response = self.client.get(self.list_url, {"iso_cc": 840})
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(len(response.data["results"]), 1)
        self.assertEqual(response.data["results"][0]["iso_cc"], 840)

    def test_filter_by_phone_code(self):
        response = self.client.get(self.list_url, {"phone_code": 33})
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(len(response.data["results"]), 1)
        self.assertEqual(response.data["results"][0]["phone_code"], 33)

    def test_filter_by_iso_cc_range(self):
        response = self.client.get(self.list_url, {"iso_cc_min": 300})
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        iso_codes = [
            item["iso_cc"]
            for item in response.data["results"]
            if item["iso_cc"]
        ]
        self.assertTrue(all(code >= 300 for code in iso_codes))

        response = self.client.get(self.list_url, {"iso_cc_max": 500})
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        iso_codes = [
            item["iso_cc"]
            for item in response.data["results"]
            if item["iso_cc"]
        ]
        self.assertTrue(all(code <= 500 for code in iso_codes))

    def test_filter_has_iso_cc(self):
        response = self.client.get(self.list_url, {"has_iso_cc": "true"})
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        for item in response.data["results"]:
            self.assertIsNotNone(item["iso_cc"])

    def test_filter_has_phone_code(self):
        response = self.client.get(self.list_url, {"has_phone_code": "true"})
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        for item in response.data["results"]:
            self.assertIsNotNone(item["phone_code"])

    def test_ordering_by_alpha_2(self):
        response = self.client.get(self.list_url, {"ordering": "alpha_2"})
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        alpha_2_codes = [item["alpha_2"] for item in response.data["results"]]
        self.assertEqual(alpha_2_codes, sorted(alpha_2_codes))

    def test_ordering_by_iso_cc_desc(self):
        response = self.client.get(self.list_url, {"ordering": "-iso_cc"})
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        iso_codes = [
            item["iso_cc"]
            for item in response.data["results"]
            if item["iso_cc"]
        ]
        self.assertEqual(iso_codes, sorted(iso_codes, reverse=True))

    def test_ordering_by_created_at(self):
        response = self.client.get(self.list_url, {"ordering": "created_at"})
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertGreater(len(response.data["results"]), 0)

    def test_search_by_alpha_2(self):
        response = self.client.get(self.list_url, {"search": "GR"})
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        found = any(
            item["alpha_2"] == "GR" for item in response.data["results"]
        )
        self.assertTrue(found)

    def test_search_by_alpha_3(self):
        response = self.client.get(self.list_url, {"search": "USA"})
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        found = any(
            item["alpha_3"] == "USA" for item in response.data["results"]
        )
        self.assertTrue(found)

    def test_search_by_phone_code(self):
        response = self.client.get(self.list_url, {"search": "33"})
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertGreater(len(response.data["results"]), 0)

    def test_create_with_invalid_alpha_2(self):
        payload = {
            "alpha_2": "invalid_alpha_2",
            "alpha_3": "CYP",
            "iso_cc": 196,
            "phone_code": 357,
            "translations": {default_language: {"name": "Cyprus"}},
        }

        response = self.client.post(self.list_url, payload, format="json")
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    def test_create_with_invalid_alpha_3(self):
        payload = {
            "alpha_2": "CY",
            "alpha_3": "INVALID",
            "iso_cc": 196,
            "phone_code": 357,
            "translations": {default_language: {"name": "Cyprus"}},
        }

        response = self.client.post(self.list_url, payload, format="json")
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    def test_create_with_duplicate_alpha_2(self):
        payload = {
            "alpha_2": "GR",
            "alpha_3": "CYP",
            "iso_cc": 196,
            "phone_code": 357,
            "translations": {default_language: {"name": "Cyprus"}},
        }

        response = self.client.post(self.list_url, payload, format="json")
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    def test_retrieve_nonexistent_country(self):
        nonexistent_url = reverse("country-detail", args=["nonexistent"])
        response = self.client.get(nonexistent_url)
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_update_nonexistent_country(self):
        nonexistent_url = reverse("country-detail", args=["nonexistent"])
        payload = {"iso_cc": 999}

        response = self.client.patch(nonexistent_url, payload, format="json")
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_alpha_codes_case_normalization(self):
        payload_lowercase = {
            "alpha_2": "cy",
            "alpha_3": "cyp",
            "iso_cc": 196,
            "phone_code": 998,
            "translations": {default_language: {"name": "Cyprus"}},
        }

        response = self.client.post(
            self.list_url, payload_lowercase, format="json"
        )
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertIn("alpha_2", response.data)
        self.assertIn("alpha_3", response.data)

        payload_uppercase = {
            "alpha_2": "CY",
            "alpha_3": "CYP",
            "iso_cc": 196,
            "phone_code": 998,
            "translations": {default_language: {"name": "Cyprus"}},
        }

        response = self.client.post(
            self.list_url, payload_uppercase, format="json"
        )
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

        self.assertEqual(response.data["alpha_2"], "CY")
        self.assertEqual(response.data["alpha_3"], "CYP")

    def test_response_includes_main_image_path(self):
        response = self.client.get(self.detail_url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertIn("main_image_path", response.data)

    def test_consistency_with_manual_serializer_instantiation(self):
        response = self.client.get(self.detail_url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        manual_serializer = CountryDetailSerializer(instance=self.country)

        for field in ["alpha_2", "alpha_3", "iso_cc", "phone_code"]:
            self.assertEqual(
                response.data[field], manual_serializer.data[field]
            )

    def test_validation_errors_consistent(self):
        payload = {
            "alpha_2": "",
            "alpha_3": "",
        }

        response = self.client.post(self.list_url, payload, format="json")
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

        self.assertIn("alpha_2", response.data)
        self.assertIn("alpha_3", response.data)



import os

from django.http import HttpRequest


def get_version_from_toml():
    with open("pyproject.toml") as toml_file:
        toml_content = toml_file.read()
        version = toml_content.split('version = "')[1].split('"\n')[0]
    return version


def metadata(request: HttpRequest):
    site_name = os.getenv("SITE_NAME", "Grooveshop")
    site_description = os.getenv("SITE_DESCRIPTION", "Grooveshop Description")
    site_keywords = os.getenv("SITE_KEYWORDS", "Grooveshop Keywords")
    site_author = os.getenv("SITE_AUTHOR", "Grooveshop Author")

    request_details = {}
    if request.user and request.user.is_superuser:
        request_details = {
            "headers": dict(request.headers),
            "cookies": request.COOKIES,
            "meta": request.META,
        }

    return {
        "VERSION": get_version_from_toml(),
        "SITE_NAME": site_name,
        "SITE_DESCRIPTION": site_description,
        "SITE_KEYWORDS": site_keywords,
        "SITE_AUTHOR": site_author,
        "REQUEST_DETAILS": request_details,
    }



from django.urls import reverse
from rest_framework.test import APITestCase

from tag.factories.tag import TagFactory
from tag.factories.tagged_item import TaggedProductFactory
from product.factories.product import ProductFactory


class TaggedItemFilterTest(APITestCase):
    def setUp(self):
        self.tag1 = TagFactory(active=True)
        self.tag2 = TagFactory(active=False)
        self.product1 = ProductFactory()
        self.product2 = ProductFactory()

        self.tagged_item1 = TaggedProductFactory(
            tag=self.tag1, content_object=self.product1
        )
        self.tagged_item2 = TaggedProductFactory(
            tag=self.tag2, content_object=self.product2
        )

    def test_tag_filter(self):
        url = reverse("tagged-item-list")

        response = self.client.get(url, {"tag": self.tag1.id})
        self.assertEqual(response.status_code, 200)
        self.assertIsInstance(response.data["results"], list)

        for item_data in response.data["results"]:
            self.assertEqual(item_data["tag"]["id"], self.tag1.id)

    def test_tag_label_filter(self):
        url = reverse("tagged-item-list")

        response = self.client.get(url, {"tag__label__icontains": "test"})
        self.assertEqual(response.status_code, 200)
        self.assertIsInstance(response.data["results"], list)

    def test_tag_active_filter(self):
        url = reverse("tagged-item-list")

        response = self.client.get(url, {"tag__active": "true"})
        self.assertEqual(response.status_code, 200)
        self.assertIsInstance(response.data["results"], list)

        for item_data in response.data["results"]:
            self.assertTrue(item_data["tag"]["active"])

        response = self.client.get(url, {"tag__active": "false"})
        self.assertEqual(response.status_code, 200)
        self.assertIsInstance(response.data["results"], list)

        for item_data in response.data["results"]:
            self.assertFalse(item_data["tag"]["active"])

    def test_content_type_filter(self):
        url = reverse("tagged-item-list")

        response = self.client.get(url, {"content_type": "product"})
        self.assertEqual(response.status_code, 200)
        self.assertIsInstance(response.data["results"], list)

        for item_data in response.data["results"]:
            self.assertEqual(item_data["content_type_name"], "product")

    def test_content_type_app_label_filter(self):
        url = reverse("tagged-item-list")

        response = self.client.get(url, {"content_type__app_label": "product"})
        self.assertEqual(response.status_code, 200)
        self.assertIsInstance(response.data["results"], list)

    def test_object_id_filter(self):
        url = reverse("tagged-item-list")

        response = self.client.get(url, {"object_id": self.product1.id})
        self.assertEqual(response.status_code, 200)
        self.assertIsInstance(response.data["results"], list)

        for item_data in response.data["results"]:
            self.assertEqual(item_data["object_id"], self.product1.id)

    def test_object_id_in_filter(self):
        url = reverse("tagged-item-list")

        object_ids = f"{self.product1.id},{self.product2.id}"
        response = self.client.get(url, {"object_id__in": object_ids})
        self.assertEqual(response.status_code, 200)
        self.assertIsInstance(response.data["results"], list)

    def test_timestamp_filters(self):
        url = reverse("tagged-item-list")

        response = self.client.get(
            url, {"created_after": "2024-01-01T00:00:00Z"}
        )
        self.assertEqual(response.status_code, 200)
        self.assertIsInstance(response.data["results"], list)

        response = self.client.get(
            url, {"created_before": "2025-12-31T23:59:59Z"}
        )
        self.assertEqual(response.status_code, 200)
        self.assertIsInstance(response.data["results"], list)

    def test_uuid_filter(self):
        url = reverse("tagged-item-list")

        response = self.client.get(url, {"uuid": str(self.tagged_item1.uuid)})
        self.assertEqual(response.status_code, 200)
        self.assertIsInstance(response.data["results"], list)

    def test_complex_filter_combinations(self):
        url = reverse("tagged-item-list")

        response = self.client.get(
            url,
            {
                "tag__active": "true",
                "content_type": "product",
                "object_id": self.product1.id,
            },
        )
        self.assertEqual(response.status_code, 200)
        self.assertIsInstance(response.data["results"], list)

    def test_filter_with_ordering(self):
        url = reverse("tagged-item-list")

        response = self.client.get(
            url, {"tag__active": "true", "ordering": "created_at"}
        )
        self.assertEqual(response.status_code, 200)
        self.assertIsInstance(response.data["results"], list)



from datetime import timedelta
from django.utils import timezone
from django.urls import reverse
from django.test import TransactionTestCase
from rest_framework.test import APIClient

from country.models import Country


class CountryFilterTest(TransactionTestCase):
    def setUp(self):
        self.client = APIClient()
        Country.objects.all().delete()

        self.now = timezone.now()

        self.usa = Country.objects.create(
            alpha_2="US", alpha_3="USA", iso_cc=840, phone_code=1, sort_order=1
        )
        self.usa.created_at = self.now - timedelta(days=365)
        self.usa.save()
        self.usa.set_current_language("en")
        self.usa.name = "United States"
        self.usa.save()
        self.usa.image_flag = "uploads/country/us.png"
        self.usa.save()

        self.uk = Country.objects.create(
            alpha_2="GB", alpha_3="GBR", iso_cc=826, phone_code=44, sort_order=2
        )
        self.uk.created_at = self.now - timedelta(days=300)
        self.uk.save()
        self.uk.set_current_language("en")
        self.uk.name = "United Kingdom"
        self.uk.save()
        self.uk.image_flag = "uploads/country/gb.png"
        self.uk.save()

        self.germany = Country.objects.create(
            alpha_2="DE", alpha_3="DEU", iso_cc=276, phone_code=49, sort_order=3
        )
        self.germany.created_at = self.now - timedelta(days=200)
        self.germany.save()
        self.germany.set_current_language("en")
        self.germany.name = "Germany"
        self.germany.save()
        self.germany.image_flag = "uploads/country/de.png"
        self.germany.save()

        self.france = Country.objects.create(
            alpha_2="FR", alpha_3="FRA", iso_cc=250, phone_code=33, sort_order=4
        )
        self.france.created_at = self.now - timedelta(days=150)
        self.france.save()
        self.france.set_current_language("en")
        self.france.name = "France"
        self.france.save()

        self.japan = Country.objects.create(
            alpha_2="JP", alpha_3="JPN", iso_cc=392, phone_code=81, sort_order=5
        )
        self.japan.created_at = self.now - timedelta(days=35)
        self.japan.save()
        self.japan.set_current_language("en")
        self.japan.name = "Japan"
        self.japan.save()
        self.japan.image_flag = "uploads/country/jp.png"
        self.japan.save()

        self.brazil = Country.objects.create(
            alpha_2="BR",
            alpha_3="BRA",
            iso_cc=None,
            phone_code=55,
            sort_order=6,
        )
        self.brazil.created_at = self.now - timedelta(days=50)
        self.brazil.save()
        self.brazil.set_current_language("en")
        self.brazil.name = "Brazil"
        self.brazil.save()
        from django.db import connection

        with connection.cursor() as cursor:
            cursor.execute(
                "UPDATE country_country SET updated_at = %s WHERE alpha_2 = %s",
                [self.now - timedelta(days=50), "BR"],
            )

        self.canada = Country.objects.create(
            alpha_2="CA",
            alpha_3="CAN",
            iso_cc=124,
            phone_code=None,
            sort_order=7,
        )
        self.canada.created_at = self.now - timedelta(days=30)
        self.canada.save()
        self.canada.set_current_language("en")
        self.canada.name = "Canada"
        self.canada.save()
        self.canada.image_flag = "uploads/country/ca.png"
        self.canada.save()

        self.unknown = Country.objects.create(
            alpha_2="XX",
            alpha_3="XXX",
            iso_cc=999,
            phone_code=999,
            sort_order=8,
        )
        self.unknown.created_at = self.now - timedelta(days=10)
        self.unknown.save()

        self.australia = Country.objects.create(
            alpha_2="AU", alpha_3="AUS", iso_cc=36, phone_code=61, sort_order=9
        )
        self.australia.created_at = self.now - timedelta(days=25)
        self.australia.save()
        self.australia.set_current_language("en")
        self.australia.name = "Australia"
        self.australia.save()
        self.australia.image_flag = "uploads/country/au.png"
        self.australia.save()

    def test_timestamp_filters(self):
        url = reverse("country-list")

        created_after = self.now - timedelta(days=40)
        response = self.client.get(
            url, {"created_at__gte": created_after.isoformat()}
        )
        self.assertEqual(response.status_code, 200)
        result_codes = [r["alpha_2"] for r in response.data["results"]]
        self.assertIn("JP", result_codes)
        self.assertIn("AU", result_codes)
        self.assertIn("CA", result_codes)
        self.assertIn("XX", result_codes)
        self.assertNotIn("BR", result_codes)

        updated_before = self.now - timedelta(days=40)
        response = self.client.get(
            url, {"updated_at__lte": updated_before.isoformat()}
        )
        self.assertEqual(response.status_code, 200)
        result_codes = [r["alpha_2"] for r in response.data["results"]]
        self.assertNotIn("JP", result_codes)
        self.assertNotIn("AU", result_codes)
        self.assertNotIn("CA", result_codes)
        self.assertNotIn("XX", result_codes)
        self.assertIn("BR", result_codes)

    def test_uuid_and_sort_order_filters(self):
        url = reverse("country-list")

        response = self.client.get(url, {"uuid": str(self.germany.uuid)})
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data["results"]), 1)
        self.assertEqual(response.data["results"][0]["alpha_2"], "DE")

        response = self.client.get(url, {"sort_order": 3})
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data["results"]), 1)
        self.assertEqual(response.data["results"][0]["alpha_2"], "DE")

        response = self.client.get(url, {"sort_order_min": 5})
        self.assertEqual(response.status_code, 200)
        result_codes = [r["alpha_2"] for r in response.data["results"]]
        self.assertIn("JP", result_codes)
        self.assertIn("BR", result_codes)
        self.assertIn("CA", result_codes)
        self.assertIn("XX", result_codes)
        self.assertIn("AU", result_codes)

    def test_code_filters(self):
        url = reverse("country-list")

        response = self.client.get(url, {"alpha_2": "US"})
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data["results"]), 1)
        self.assertEqual(response.data["results"][0]["alpha_2"], "US")

        response = self.client.get(url, {"alpha_2__icontains": "U"})
        self.assertEqual(response.status_code, 200)
        result_codes = [r["alpha_2"] for r in response.data["results"]]
        self.assertIn("US", result_codes)
        self.assertIn("AU", result_codes)

        response = self.client.get(url, {"alpha_3": "GBR"})
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data["results"]), 1)
        self.assertEqual(response.data["results"][0]["alpha_3"], "GBR")

        response = self.client.get(url, {"alpha_3__icontains": "RA"})
        self.assertEqual(response.status_code, 200)
        result_codes = [r["alpha_2"] for r in response.data["results"]]
        self.assertIn("FR", result_codes)
        self.assertIn("BR", result_codes)

    def test_iso_and_phone_code_filters(self):
        url = reverse("country-list")

        response = self.client.get(url, {"iso_cc": 840})
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data["results"]), 1)
        self.assertEqual(response.data["results"][0]["alpha_2"], "US")

        response = self.client.get(url, {"iso_cc_min": 200, "iso_cc_max": 300})
        self.assertEqual(response.status_code, 200)
        result_codes = [r["alpha_2"] for r in response.data["results"]]
        self.assertIn("DE", result_codes)
        self.assertIn("FR", result_codes)

        response = self.client.get(url, {"phone_code": 44})
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data["results"]), 1)
        self.assertEqual(response.data["results"][0]["alpha_2"], "GB")

        response = self.client.get(
            url, {"phone_code_min": 40, "phone_code_max": 60}
        )
        self.assertEqual(response.status_code, 200)
        result_codes = [r["alpha_2"] for r in response.data["results"]]
        self.assertIn("GB", result_codes)
        self.assertIn("DE", result_codes)
        self.assertIn("BR", result_codes)

    def test_name_filters(self):
        url = reverse("country-list")

        response = self.client.get(url, {"name": "united"})
        self.assertEqual(response.status_code, 200)
        result_codes = [r["alpha_2"] for r in response.data["results"]]
        self.assertIn("US", result_codes)
        self.assertIn("GB", result_codes)

        response = self.client.get(url, {"name__exact": "Japan"})
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data["results"]), 1)
        self.assertEqual(response.data["results"][0]["alpha_2"], "JP")

        response = self.client.get(url, {"name__startswith": "Ger"})
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data["results"]), 1)
        self.assertEqual(response.data["results"][0]["alpha_2"], "DE")

    def test_special_filters(self):
        url = reverse("country-list")

        response = self.client.get(url, {"multiple_codes": "US,GBR,JP"})
        self.assertEqual(response.status_code, 200)
        result_codes = [r["alpha_2"] for r in response.data["results"]]
        self.assertIn("US", result_codes)
        self.assertIn("GB", result_codes)
        self.assertIn("JP", result_codes)
        self.assertEqual(len(result_codes), 3)

        response = self.client.get(url, {"is_eu": "true"})
        self.assertEqual(response.status_code, 200)
        result_codes = [r["alpha_2"] for r in response.data["results"]]
        self.assertIn("DE", result_codes)
        self.assertIn("FR", result_codes)
        self.assertNotIn("US", result_codes)
        self.assertNotIn("GB", result_codes)

        response = self.client.get(url, {"has_name": "true"})
        self.assertEqual(response.status_code, 200)
        result_codes = [r["alpha_2"] for r in response.data["results"]]
        self.assertIn("US", result_codes)
        self.assertIn("GB", result_codes)
        self.assertIn("DE", result_codes)
        self.assertIn("FR", result_codes)
        self.assertIn("JP", result_codes)
        self.assertIn("BR", result_codes)
        self.assertIn("CA", result_codes)
        self.assertIn("AU", result_codes)
        self.assertNotIn("XX", result_codes)

    def test_camel_case_filters(self):
        url = reverse("country-list")

        created_after = self.now - timedelta(days=60)
        response = self.client.get(
            url,
            {
                "createdAfter": created_after.isoformat(),
                "hasIsoCC": "true",
                "hasFlagImage": "true",
            },
        )
        self.assertEqual(response.status_code, 200)

        result_codes = [r["alpha_2"] for r in response.data["results"]]
        self.assertIn("JP", result_codes)
        self.assertIn("CA", result_codes)
        self.assertIn("AU", result_codes)
        self.assertNotIn("BR", result_codes)

        response = self.client.get(url, {"phoneCode": 1})
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data["results"]), 1)
        self.assertEqual(response.data["results"][0]["alpha_2"], "US")

    def test_complex_filter_combinations(self):
        url = reverse("country-list")

        response = self.client.get(
            url, {"is_eu": "true", "has_all_data": "true", "sort_order_max": 5}
        )
        self.assertEqual(response.status_code, 200)
        result_codes = [r["alpha_2"] for r in response.data["results"]]
        self.assertIn("DE", result_codes)
        self.assertNotIn("FR", result_codes)

    def test_filter_with_ordering(self):
        url = reverse("country-list")

        response = self.client.get(
            url, {"has_name": "true", "ordering": "sort_order"}
        )
        self.assertEqual(response.status_code, 200)

        results = response.data["results"]
        self.assertEqual(results[0]["alpha_2"], "US")
        self.assertEqual(results[1]["alpha_2"], "GB")
        self.assertEqual(results[2]["alpha_2"], "DE")
        self.assertEqual(results[3]["alpha_2"], "FR")

    def tearDown(self):
        Country.objects.all().delete()
